proc template;

    /********************************************************/
    /********************************************************/
    /********************************************************/
    /*  STATGRAPH - XML output to render statistical graphs */
    /*  in HTML, etc.                                       */
    /********************************************************/
    /********************************************************/
    /********************************************************/

    /* version */
    %let _stg_version_ = '"9.3"';    /* StatGraph */
    %let _g_version_ = '"9.3"';      /* SAS/Graph */
    %let _srx_version_ = '"1.2"';    /* SASReport */

    /* Generate a list */
    %macro StatListMacro(evtName, list);
        break / if ^exists(&list);
        ndent;
        put "<&evtName>" CR;
        ndent;
        iterate &list;
        do / while _value_;
            do / if cmp(_value_, "___EMPTY___");
               put "<Value></Value>" CR;
            else;
               put "<Value>" _value_ "</Value>" CR;
            done;
           next &list;
        done;
        xdent;
        put "</&evtName>" CR;
        xdent;

        unset &list;    /* reset it for next use */
    %mend;

    /* Generate a typed value list */
    %macro StatTypedListMacro(evtName, list);
        break / if ^exists(&list);
        ndent;
        put "<&evtName>" CR;
        ndent;
        iterate &list;  /* This is a hashmap: _name_ -> _value_ */
        do / while _name_;
            put "<Value ";
            put 'type="' _value_ '"'/ if ^cmp(_value_, ' ');
            put ">" ;
            do / if cmp(_value_, 'char');
                put ktrim(_name_) ;  /* S0696176 */
            else;
                put _name_;
            done;
            put "</Value>" CR;
            next &list;
        done;
        xdent;
        put "</&evtName>" CR;
        xdent;

        unset &list;    /* reset it for next use */
    %mend;

    /* Generate a variable list */
    %macro StatVariableListMacro(evtName, list);
        break / if ^exists(&list);
        ndent;
        put "<&evtName>" CR;
        ndent;
        iterate &list;
        do / while _value_;
            put "<Value>";
            put $models[_value_] anchor "#" _value_;
            put "</Value>" CR;
           next &list;
        done;
        xdent;
        put "</&evtName>" CR;
        xdent;
    %mend;

    /* Script attrName=model#var if var present */
    %macro putModelVar(var, attrName);
       put " &attrName=""" $models[&var] anchor "#" &var """" / if exists(&var);
    %mend putModelVar;

    /* Handle ploterror at finish event */
    %macro plotErrorFinish;
        do / if $ploterror;
            unset $ploterror;
            trigger StatPlotError;
            break;
        done;
    %mend plotErrorFinish;

    /* Handle missing required args for components at finish event */
    %macro missingReqArgsHandler;
        do / if $missingReqArgs;
            unset $missingReqArgs;
                /* Log it as a warning. The extra " " works around a bug.
                 * (Eric Gebhart: it confuses the look ahead for the connection
                 * between labels and variables.)
                 * ReqArgsMissingMsg will only be set for SGRENDER, data null */
            putlog "%2z" " " @ReqArgsMissingMsg / if @ReqArgsMissingMsg;
            break;
        done;
    %mend missingReqArgsHandler;

    %macro tagsets_statgraph;

    %if  %substr(&sysver,1,1) = 8 %then %do;
       /* not in version 8 */
       %end;

    %else %do;

    define tagset tagsets.statgraph;
        private;
        notes "This is the StatGraph XML definition";

        /* +----------------------------------------------+
           | ALF/2002-01-30    Re: S0128698               |
           | We now parent the engine formatting routine  |
           | for graphics stuff. see file XMLfmt.tpl for  |
           | details. this definition will be shared by   |
           | the XML engine tagsets in file XMLeng.tpl    |
           +----------------------------------------------+ */

        parent = tagsets.sasFMT;

        %let  map =<>%nrstr(&)%str(%')%str(%");
        map="&map";
        mapsub = '/&lt;/&gt;/&amp;/&#39;/&quot;/';
        nobreakspace = ' ';
        split = '<br/>';
        output_type = 'xml';
        indent=2;

        define event initUnicodeMap;
            set $unicodeMap["ALPHA" ] "03B1";
            set $unicodeMap["ALPHA_U" ] "0391";
            set $unicodeMap["BETA" ] "03B2";
            set $unicodeMap["BETA_U" ] "0392";
            set $unicodeMap["GAMMA" ] "03B3";
            set $unicodeMap["GAMMA_U" ] "0393";
            set $unicodeMap["DELTA" ] "03B4";
            set $unicodeMap["DELTA_U" ] "0394";
            set $unicodeMap["EPSILON" ] "03B5";
            set $unicodeMap["EPSILON_U" ] "0395";
            set $unicodeMap["ZETA" ] "03B6";
            set $unicodeMap["ZETA_U" ] "0396";
            set $unicodeMap["ETA" ] "03B7";
            set $unicodeMap["ETA_U" ] "0397";
            set $unicodeMap["THETA" ] "03B8";
            set $unicodeMap["THETA_U" ] "0398";
            set $unicodeMap["IOTA" ] "03B9";
            set $unicodeMap["IOTA_U" ] "0399";
            set $unicodeMap["KAPPA" ] "03BA";
            set $unicodeMap["KAPPA_U" ] "039A";
            set $unicodeMap["LAMBDA" ] "03BB";
            set $unicodeMap["LAMBDA_U" ] "039B";
            set $unicodeMap["MU" ] "03BC";
            set $unicodeMap["MU_U" ] "039C";
            set $unicodeMap["NU" ] "03BD";
            set $unicodeMap["NU_U" ] "039D";
            set $unicodeMap["XI" ] "03BE";
            set $unicodeMap["XI_U" ] "039E";
            set $unicodeMap["OMICRON" ] "03BF";
            set $unicodeMap["OMICRON_U" ] "039F";
            set $unicodeMap["PI" ] "03C0";
            set $unicodeMap["PI_U" ] "03A0";
            set $unicodeMap["RHO" ] "03C1";
            set $unicodeMap["RHO_U" ] "03A1";
            set $unicodeMap["SIGMA" ] "03C3";
            set $unicodeMap["SIGMA_U" ] "03A3";
            set $unicodeMap["TAU" ] "03C4";
            set $unicodeMap["TAU_U" ] "03A4";
            set $unicodeMap["UPSILON" ] "03C5";
            set $unicodeMap["UPSILON_U" ] "03A5";
            set $unicodeMap["PHI" ] "03C6";
            set $unicodeMap["PHI_U" ] "03A6";
            set $unicodeMap["CHI" ] "03C7";
            set $unicodeMap["CHI_U" ] "03A7";
            set $unicodeMap["PSI" ] "03C8";
            set $unicodeMap["PSI_U" ] "03A8";
            set $unicodeMap["OMEGA" ] "03C9";
            set $unicodeMap["OMEGA_U" ] "03A9";
            set $unicodeMap["BAR" ] "0305";
            set $unicodeMap["BAR2" ] "033F";
            set $unicodeMap["HAT" ] "0302";
            set $unicodeMap["TILDE" ] "0303";
            set $unicodeMap["PRIME" ] "00B4";
        end;

        define event initStyleMap;
            set $elementMap['twocolorramp' ] 'TwoColorRamp';
            set $elementMap['twocoloraltramp' ] 'TwoColorAltRamp';
            set $elementMap['threecolorramp'] 'ThreeColorRamp';
            set $elementMap['threecoloraltramp'] 'ThreeColorAltRamp';
            set $elementMap['graphaxislines'] 'AxisLines';
            set $elementMap['graphdatadefault'] 'GraphDataStyleDefault';
            set $elementMap['graphgridlines'] 'GridLines';
            set $elementMap['graphoutlines'] 'Outlines';
            set $elementMap['graphborderlines'] 'BorderLines';
            set $elementMap['graphtitletext'] 'TitleText';
            set $elementMap['graphfootnotetext'] 'FootnoteText';
            set $elementMap['graphdatatext'] 'DataText';
            set $elementMap['graphlabeltext'] 'LabelText';
            set $elementMap['graphlabel2text'] 'Label2Text';
            set $elementMap['graphvaluetext'] 'ValueText';
            set $elementMap['graphunicodetext'] 'UnicodeText';
            set $elementMap['graphbackground'] 'BackFill';
            set $elementMap['graphwalls'] 'WallFill';
            set $elementMap['graphfloor'] 'FloorFill';
            set $elementMap['graphlegendbackground'] 'LegendFill';
            set $elementMap['graphheaderbackground'] 'HeaderFill';
            set $elementMap['graphoutlier'] 'Outlier';
            set $elementMap['graphfit'] 'Fit';
            set $elementMap['graphfit2'] 'Fit2';
            set $elementMap['graphprediction'] 'Prediction';
            set $elementMap['graphconfidence'] 'Confidence';
            set $elementMap['graphconfidence2'] 'Confidence2';
            set $elementMap['graphpredictionlimits'] 'PredictionLimits';
            set $elementMap['graphreference'] 'Reference';
            set $elementMap['grapherror'] 'Error';
            set $elementMap['graphboxmedian'] 'BoxMedian';
            set $elementMap['graphboxmean'] 'BoxMean';
            set $elementMap['graphboxwhisker'] 'BoxWhisker';
            set $elementMap['graphannoline'] 'AnnoLine';
            set $elementMap['graphannotext'] 'AnnoText';
            set $elementMap['graphannoshape'] 'AnnoShape';
            set $elementMap['graphselection'] 'Selection';
            set $elementMap['graphconnectline'] 'ConnectLine';
            set $elementMap['graphmissing'] 'Missing';
            set $elementMap['graphcontrollimits'] 'ControlLimits';
            set $elementMap['graphruntest'] 'RunTest';
            set $elementMap['graphstars'] 'Stars';
            set $elementMap['graphclipping'] 'Clipping';
            set $elementMap['graphblock'] 'Block';
            set $elementMap['graphaltblock'] 'AltBlock';
            set $elementMap['graphother'] 'Other';
            set $elementMap['graphinitial'] 'Initial';
            set $elementMap['graphfinal'] 'Final';
            set $elementMap['graphoverflow'] 'Overflow';
            set $elementMap['graphunderflow'] 'Underflow';
            set $elementMap['graphcutline'] 'CutLine';

            set $attributeMap['linestyle'] 'lineStyle';
            set $attributeMap['linethickness'] 'lineThickness';
            set $attributeMap['markersymbol'] 'markerSymbol';
            set $attributeMap['markersize'] 'markerSize';
            set $attributeMap['startcolor'] 'fillGradientStartColor';
            set $attributeMap['neutralcolor'] 'fillGradientNeutralColor';
            set $attributeMap['endcolor'] 'fillGradientEndColor';
            set $attributeMap['transparency'] 'transparency';
            set $attributeMap['fontweight'] 'fontWeight';
            set $attributeMap['fontstyle'] 'fontStyle';
            set $attributeMap['fontfamily'] 'fontFamily';
            set $attributeMap['fontsize'] 'fontSize';
            set $attributeMap['frameborder'] 'frameBorder';
            set $attributeMap['displayopts'] 'displayOpts';
            set $attributeMap['capstyle'] 'capStyle';
            set $attributeMap['connect'] 'connect';

        end;

      define event unicode ;
         Notes "Unicode function inserts unicode values. ";
         start:
            unset $newvalue;
            eval $temp value ;
            set  $temp strip($temp);
            set  $temp upcase($temp);

            do / if  $unicodeMap[$temp] ;
                   set $newvalue $unicodeMap[$temp] ;
                   put '&#x' $newvalue ';';
            else;
                set $sglq "'";
                set $dblq '"';
                do / if (index($temp, "X") > 0);
                     do / if (index($temp, $sglq) > 0);
                          set $newvalue scan($temp, 1, "'");
                     else;
                         do / if (index($temp, $dblq) > 0);
                             set $newvalue scan($temp, 1, '"');
                         done;
                     done;
                     do / if $newvalue;
                         put '&#x' $newvalue ';';
                     done;
               done ;
          done ;
    end;

        define event StatPutData;
          set $databucket ref_id;
          putstream $databucket;
          unset $databucket;
        end;

        define event StatDataModel;
          start:
             ndent;
             put '<DataModel';
             put ' name="StatGraphDataModel';
             put anchor '"';
             put '>' CR;
          finish:
             put '</DataModel>' CR;
             xdent;
        end;

        /* ndents seem to be global - not per stream. This causes accumulated
         * indents for nested events when there are multiple models.
         */
        define event StatGraphData;
          start:
             set $databucket ref_id;
             open $databucket / if $databucket;
             ndent;
             put '<IQData type="2DEmbedded"';
             put ' name="';
             do / if ref_id;
                put ref_id;
             else;
                put "StatGraphData";
             done;
             put anchor;
             put '">' CR;
             ndent;
             put "<EmbeddedData>" NL;
             do / if $databucket;
                close;
                unset $databucket;
             done;
          finish:
             set $databucket ref_id;
             open $databucket / if $databucket;
             put "</EmbeddedData>" NL;
             xdent;
             put "</IQData>" CR;
             do / if $mapModel;
                do / if cmp($mapModel, ref_id);
                    put "<EmbeddedGeometryProvider";
                    put ' name="SASMapData"';
                    put ' dataId="' $mapModel anchor '"';
                    put ' />' CR;
                else;
                    do / if ^ref_id;
                      put "<EmbeddedGeometryProvider";
                      put ' name="SASMapData"';
                      put ' dataId="StatGraphData' anchor '"';
                      put ' />' CR;
                    done;
                done;
             done;
             xdent;
             do / if $databucket;
                close;
                unset $databucket;
             done;
        end;

        define event StatVariableList;
           start:
             set $databucket ref_id;
             open $databucket / if $databucket;
             put "<Variables>" NL;
             do / if $databucket;
                close;
                unset $databucket;
             done;
           finish:
             set $databucket ref_id;
             open $databucket / if $databucket;
             put "</Variables>" NL;
             do / if $databucket;
                close;
                unset $databucket;
             done;
        end;

        define event StatVarBucket;
            do / if ref_id;
               do / if ^exists(@RealName);
                   set $models[name] ref_id;
               done;
            else;
               set $models[name] "StatGraphData";
            done;
        end;

        define event StatVariable;
          start:
            trigger StatVarBucket;
            set $databucket ref_id;
            open $databucket / if $databucket;
            ndent;
            put "<Variable";
            putq " name=" name;
            putq " label=" label;
            putq " type=" type;
            putq " valuesCount=" @Nobs;
            putq " SASFormat=" @Format;
            putq " encoding=" @Encoding;
            putq " length=" @Length;
            put "/>" CR;
            xdent;
            do / if $databucket;
               close;
               unset $databucket;
            done;
        end;

        define event StatValuesList;
          start:
            set $databucket ref_id;
            open $databucket / if $databucket;
            ndent;
            put "<ValuesList";
            putq " valuesCount=" @Nobs;
            put ">" CR;
            do / if $databucket;
               close;
               unset $databucket;
            done;
          finish:
            set $databucket ref_id;
            open $databucket / if $databucket;
            put "</ValuesList>" CR;
            xdent;
            do / if $databucket;
               close;
               unset $databucket;
            done;
        end;

        define event StatAnnotateModel;
             set $annotateModel ref_id;
        end;

        define event StatMapModel;
             set $mapModel ref_id;
        end;

        define event StatXMLDecl;
            put '<?xml version="1.0"';
            putq ' encoding=' ENCODING / if exists(ENCODING);
            put ' ?>' CR;
        end;

        define event StatDocument;
          start:
            trigger StatXMLDecl;
            put '<SASReport version=' &_srx_version_ ' xmlns:style="http://www.sas.com/sasreportmodel/styles">' CR;
          finish:
            put "</SASReport>" CR;
        end;

        define event StatDataList;
          start:
            trigger StatDocument start;
            put '<Data version=' &_stg_version_ '>' NL;
          finish:
            put "</Data>" NL;
            trigger StatDocument finish;
        end;

/* We need this event for the SASReport tagset */
        define event StatGraphView;
          start:
            ndent;
            trigger initUnicodeMap / if ^$unicodeMap;
            put "<View>" CR;
          finish:
            put "</View>" CR;
            unset $annotateModel; /* unset the anno model name */
            unset $mapModel; /* unset the map model name */
            unset $models;      /* clear data models */
            xdent;
        end;

/***********************************************************************/
/*                   Graph appearance scheme events.                   */
/***********************************************************************/

        define event StatStyleScheme;
          start:
            ndent;
            trigger initStyleMap / if ^$elementMap;
            put "<style:Scheme";
            put ' name="StatGraphScheme';
            put anchor '"';
            put ' version=' &_stg_version_ '>' CR;
          finish:
            put "</style:Scheme>" CR;
            xdent;
        end;

        define event StatGraphScheme;
          start:
            pure_style;
            style=Graph;
            unset $StyleAttrPriority;
            set $StyleAttrPriority ATTRPRIORITY / if ATTRPRIORITY;
            ndent;
            put '<style:Style name="StatGraphStyle';
            put anchor '"';
            putq " styleAttrPriority=" $StyleAttrPriority / if $StyleAttrPriority;
            put ">" CR;
          finish:
            put "</style:Style>" CR;
            xdent;
        end;

        define event StatLogicalFonts;
          start:
            ndent;
            put "<LogicalFonts>" CR;

          finish:
            put "</LogicalFonts>" CR;
            xdent;
        end;

        define event StatFontMapItem;
            ndent;
            put "<" name ">"  value "</" name ">" CR;
            xdent;
        end;

        define event StatStyleColAltFillTextColors;
            putq " color=" COLOR;
            putq " altColor=" CONTRASTCOLOR;
            putq " fillColor=" COLOR;
            putq " textColor=" CONTRASTCOLOR;
        end;

        define event StatDataStyle;
            pure_style;
            /* style= is passed in as OA_STYLE */
            ndent;
            put '<style:Style context="GraphDataStyle' ref_id '"';

            trigger StatStyleColAltFillTextColors;
            /* No support for changing transparency */
            /* No support for changing font attrs */

            putq " lineStyle=" LINESTYLE;
            putq " lineColor=" CONTRASTCOLOR;
            /* No support for changing lineThickness */

            putq " markerSymbol=" MARKERSYMBOL;
            putq " markerColor=" CONTRASTCOLOR;
            /* No support for changing markerSize */

            putq " fillPattern=" FILLPATTERN;

            put "/>" NL;
            xdent;
        end;


        define event StatLineThickness;
            putq " lineThickness=" LINETHICKNESS / if exists(LINETHICKNESS);
            putq " lineThickness=" OUTPUTWIDTH / if ^exists(LINETHICKNESS);
        end;

         define event StatDataDefaultScheme;
            pure_style;
            style=GraphDataDefault;
            break / if ^any(COLOR, CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="GraphDataStyleDefault" ';

            trigger StatStyleColAltFillTextColors;

            trigger StatStyleLineAttrs;
            trigger StatStyleMarkerAttrs;

            trigger StatStyleCRampAttrs;

            put "/>" NL;
            xdent;
        end;

        define event StatLinePattern;
            break / if ^exists($pattern);
             put '"1"' / breakif cmp($pattern,"SOLID");
             put '"2"' / breakif cmp($pattern,"SHORTDASH");
             put '"4"' / breakif cmp($pattern,"MEDIUMDASH");
             put '"5"' / breakif cmp($pattern,"LONGDASH");
             put '"8"' / breakif cmp($pattern,"MEDIUMDASHSHORTDASH");
             put '"14"' / breakif cmp($pattern,"DASHDASHDOT");
             put '"15"' / breakif cmp($pattern,"DASHDOTDOT");
             put '"20"' / breakif cmp($pattern,"DASH");
             put '"26"' / breakif cmp($pattern,"LONGDASHSHORTDASH");
             put '"34"' / breakif cmp($pattern,"DOT");
             put '"35"' / breakif cmp($pattern,"THINDOT");
             put '"41"' / breakif cmp($pattern,"SHORTDASHDOT");
             put '"42"' / breakif cmp($pattern,"MEDIUMDASHDOTDOT");
             putq $pattern;
        end;

        define event StatFindNeutralColor;

          set $csr substr(STARTCOLOR, 2, 2);
          set $csg substr(STARTCOLOR, 4, 2);
          set $csb substr(STARTCOLOR, 6, 2);
          set $cer substr(ENDCOLOR, 2, 2);
          set $ceg substr(ENDCOLOR, 4, 2);
          set $ceb substr(ENDCOLOR, 6, 2);

          eval $nsr inputn($csr, "hex2.");
          eval $nsg inputn($csg, "hex2.");
          eval $nsb inputn($csb, "hex2.");
          eval $ner inputn($cer, "hex2.");
          eval $neg inputn($ceg, "hex2.");
          eval $neb inputn($ceb, "hex2.");

          eval $fr ($ner + (($nsr-$ner) / 2));
          eval $fg ($neg + (($nsg-$neg) / 2));
          eval $fb ($neb + (($nsb-$neb) / 2));

          set $nr putn($fr, "hex2.");
          set $ng putn($fg, "hex2.");
          set $nb putn($fb, "hex2.");

          put '"#' $nr $ng $nb '"';

          unset $csr;
          unset $csg;
          unset $csb;
          unset $cer;
          unset $ceg;
          unset $ceb;
          unset $fr;
          unset $fg;
          unset $fb;
          unset $nr;
          unset $ng;
          unset $nb;
        end;

        define event StatStyleCRampAttrs;
          break / if ^exists( STARTCOLOR , ENDCOLOR );
          putq " fillGradientStartColor=" STARTCOLOR;
          do / if NEUTRALCOLOR;
             putq " fillGradientNeutralColor=" NEUTRALCOLOR;
          else;
             put " fillGradientNeutralColor=";
             trigger StatFindNeutralColor;
          done;
          putq " fillGradientEndColor=" ENDCOLOR;
        end;

        define event StatTwoColorRampScheme;
          pure_style;
          style=TwoColorRamp;
            break / if ^exists( STARTCOLOR , ENDCOLOR );
          ndent;
          put '<style:Style context="TwoColorRamp"';
          trigger StatStyleCRampAttrs;
          put "/>" NL;
          xdent;
        end;

        define event StatTwoColorAltRampScheme;
          pure_style;
          style=TwoColorAltRamp;
            break / if ^exists( STARTCOLOR , ENDCOLOR );
          ndent;
          put '<style:Style context="TwoColorAltRamp"';
          trigger StatStyleCRampAttrs;
          put "/>" NL;
          xdent;
        end;

        define event StatThreeColorRampScheme;
          pure_style;
          style=ThreeColorRamp;
            break / if ^exists( STARTCOLOR , ENDCOLOR );
          ndent;
          put '<style:Style context="ThreeColorRamp"';
          trigger StatStyleCRampAttrs;
          put "/>" NL;
          xdent;
        end;

       define event StatThreeColorAltRampScheme;
         pure_style;
         style=ThreeColorAltRamp;
            break / if ^exists( STARTCOLOR , ENDCOLOR );
         ndent;
         put '<style:Style context="ThreeColorAltRamp"';
         trigger StatStyleCRampAttrs;
         put "/>" NL;
         xdent;
        end;

        define event StatStyleLineAttrs;
            putq " lineColor=" CONTRASTCOLOR;
            putq " lineStyle=" LINESTYLE;
            trigger StatLineThickness;
        end;

        define event StatStyleMarkerAttrs;
            putq " markerColor=" CONTRASTCOLOR;
            putq " markerSymbol=" MARKERSYMBOL;
            putq " markerSize=" MARKERSIZE;
        end;

        define event StatAxisLineScheme;
            break / if ^any(CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                                LINETHICKNESS);
            ndent;
            put '<style:Style context="AxisLines"';
            trigger StatStyleLineAttrs;
            putq " tickDisplay=" TICKDISPLAY;
            put "/>" NL;
            xdent;
          style = GraphAxisLines;
          pure_style;
        end;

        define event StatCutLineScheme;
          break / if ^any(CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS);
          ndent;
          put '<style:Style context="CutLine"';
          trigger StatStyleLineAttrs;
          put "/>" NL;
          xdent;
          style = GraphCutLine;
          pure_style;
        end;

        define event StatGridLineScheme;
            break / if ^any(CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS, DISPLAYOPTS);
            ndent;
            put '<style:Style context="GridLines"';
            trigger StatStyleLineAttrs;
            putq " displayOpts=" DISPLAYOPTS;
            put "/>" NL;
            xdent;
         style = GraphGridLines;
         pure_style;
        end;

        define event StatReferenceScheme;
            break / if ^any(CONTRASTCOLOR, COLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="Reference" ';

            trigger StatStyleColAltFillTextColors;
            trigger StatStyleLineAttrs;
            trigger StatStyleMarkerAttrs;
            put "/>" NL;
            xdent;
         style = GraphReference;
         pure_style;
        end;

        define event StatOutlineScheme;
            break / if ^any(CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                                    LINETHICKNESS);
            ndent;
            put '<style:Style context="Outlines" ';
            trigger StatStyleLineAttrs;
            put "/>" NL;
            xdent;
         style = GraphOutlines;
         pure_style;
        end;

        define event StatBorderLineScheme;
            break / if ^any(CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                                LINETHICKNESS);
            ndent;
            put '<style:Style context="BorderLines"';
            trigger StatStyleLineAttrs;
            put "/>" NL;
            xdent;
         style = GraphBorderLines;
         pure_style;
        end;

        define event StatStyleFontAttrs;
            putq " fontWeight=" FONT_WEIGHT;
            putq " fontStyle=" FONT_STYLE;
            putq " fontFamily=" FONT_FACE;
            putq " fontSize=" FONT_SIZE;
        end;

        define event StatStyleTextAttrs;
            putq " textColor=" COLOR;
            trigger StatStyleFontAttrs;
        end;

        define event StatLabelTextScheme;
            style = GraphLabelText;
            pure_style;
            break / if ^any(COLOR, FONT_FACE, FONT_STYLE, FONT_WEIGHT,
                                                    FONT_SIZE);
            ndent;
            put '<style:Style context="LabelText"';
            trigger StatStyleTextAttrs;
            put "/>" NL;
            xdent;
        end;

        define event StatLabel2TextScheme;
            style = GraphLabel2Text;
            pure_style;
            break / if ^any(COLOR, FONT_FACE, FONT_STYLE, FONT_WEIGHT,
                                                    FONT_SIZE);
            ndent;
            put '<style:Style context="Label2Text"';
            trigger StatStyleTextAttrs;
            put "/>" NL;
            xdent;
        end;

        define event StatValueTextScheme;
            style = GraphValueText;
            pure_style;
            break / if ^any(COLOR, FONT_FACE, FONT_STYLE, FONT_WEIGHT,
                                            FONT_SIZE);
            ndent;
            put '<style:Style context="ValueText"';
            trigger StatStyleTextAttrs;
            put "/>" NL;
            xdent;
        end;

        define event StatUnicodeTextScheme;
            style = GraphUnicodeText;
            pure_style;
            break / if ^any( FONT_FACE );
            ndent;
            put '<style:Style context="UnicodeText"';
            putq " fontFamily=" FONT_FACE;
            put "/>" NL;
            xdent;
        end;

        define event StatTitleTextScheme;
            style = GraphTitleText;
            pure_style;
            break / if ^any(COLOR, FONT_FACE, FONT_STYLE, FONT_WEIGHT,
                                                FONT_SIZE);
            ndent;
            put '<style:Style context="TitleText"';
            trigger StatStyleTextAttrs;
           put "/>" NL;
            xdent;
        end;

        define event StatFootnoteTextScheme;
            style = GraphFootnoteText;
            pure_style;
            break / if ^any(COLOR, FONT_FACE, FONT_STYLE, FONT_WEIGHT,
                                                FONT_SIZE);
            ndent;
            put '<style:Style context="FootnoteText"';
            trigger StatStyleTextAttrs;
            put "/>" NL;
            xdent;
        end;

        define event StatDataTextScheme;
            style = GraphDataText;
            pure_style;
            break / if ^any(COLOR, FONT_FACE, FONT_STYLE, FONT_WEIGHT,
                                            FONT_SIZE);
            ndent;
            put '<style:Style context="DataText"';

            trigger StatStyleTextAttrs;
            put "/>" NL;
            xdent;
        end;

        define event StatBackFillScheme;
            break / if ^any(COLOR, STARTCOLOR, ENDCOLOR);
            ndent;
            put '<style:Style context="BackFill"';
            putq " fillColor=" COLOR;

                /* Needed for banker style */
            putq " fillColor=" STARTCOLOR / if ^exists( COLOR );
            putq " fillColor=" ENDCOLOR / if ^any( COLOR, STARTCOLOR );

            put "/>" NL;
            xdent;
         style = GraphBackground;
         pure_style;
        end;

        /* Why does a fill scheme need line attrs?? */
        define event StatWallFillScheme;
            break / if ^any(COLOR, CONTRASTCOLOR, LINESTYLE, LINETHICKNESS,
                                            FRAMEBORDER);
            ndent;
            put '<style:Style context="WallFill"';

            putq " fillColor=" COLOR;
            /* No support for transparency */

            trigger StatStyleLineAttrs;

            /* S0502215 - script only if present in style */
            put ' frameBorder="true"' / if cmp(FRAMEBORDER, "yes");
            put ' frameBorder="false"' / if cmp(FRAMEBORDER, "no");

           put "/>" NL;
            xdent;
         style = GraphWalls;
         pure_style;
        end;


        define event StatFloorFillScheme;
            break / if ^exist( COLOR );
            ndent;
            put '<style:Style context="FloorFill"';
            putq " fillColor=" COLOR;
            put "/>" NL;
            xdent;
          style = GraphFloor;
          pure_style;
        end;

        define event StatStyleFillAttrs;
            putq " fillColor=" COLOR;
            putq " transparency=" TRANSPARENCY / if exist(TRANSPARENCY) and
                                        0 <= inputn(TRANSPARENCY, '4.2') <= 1;
        end;

        define event StatLegendFillScheme;
            break / if ^exist( COLOR );
            ndent;
            put '<style:Style context="LegendFill"';
            trigger StatStyleFillAttrs;
            put "/>" NL;
            xdent;
          style = GraphLegendBackground;
          pure_style;
        end;

        define event StatHeaderFillScheme;
            break / if ^exist( COLOR );
            ndent;
            put '<style:Style context="HeaderFill"';
            trigger StatStyleFillAttrs;
            put "/>" NL;
            xdent;
          style = GraphHeaderBackground;
          pure_style;
        end;

        define event StatStyleColorFillLineMarkerAttrs;
            putq " color=" COLOR;
            putq " altColor=" CONTRASTCOLOR;
            trigger StatStyleFillAttrs;
            putq " textColor=" CONTRASTCOLOR;
            trigger StatStyleLineAttrs;
            trigger StatStyleMarkerAttrs;
        end;

        define event StatOutlierScheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                            LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Outlier" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
         style = GraphOutlier;
         pure_style;
        end;

        define event StatOtherScheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                            LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Other" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
        style = GraphOther;
        pure_style;
        end;

        define event StatInitialScheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                            LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Initial" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
        style = GraphInitial;
        pure_style;
        end;

        define event StatFinalScheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                            LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Final" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
        style = GraphFinal;
        pure_style;
        end;

        define event StatOverflowScheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                            LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Overflow" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
        style = GraphOverflow;
        pure_style;
        end;

        define event StatUnderflowScheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                            LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Underflow" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
        style = GraphUnderflow;
        pure_style;
        end;



        define event StatFitScheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                            LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Fit" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
         style = GraphFit;
         pure_style;
        end;

        define event StatFit2Scheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                            LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Fit2" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
         style = GraphFit2;
         pure_style;
        end;

        define event StatPredictionScheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                            LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Prediction" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
         style = GraphPrediction;
         pure_style;
        end;

        define event StatConfidenceScheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                                LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Confidence" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
         style = GraphConfidence;
         pure_style;
        end;

        define event StatConfidence2Scheme;
            break / if ^any(CONTRASTCOLOR,COLOR,LINESTYLE,OUTPUTWIDTH,
                                LINETHICKNESS,MARKERSYMBOL,MARKERSIZE);
            ndent;
            put '<style:Style context="Confidence2" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
         style = GraphConfidence2;
         pure_style;
        end;

        define event StatPredictionLimitsScheme;
            break / if ^any(CONTRASTCOLOR, COLOR, LINESTYLE, OUTPUTWIDTH,
                                LINETHICKNESS, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="PredictionLimits" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
         style = GraphPredictionLimits;
         pure_style;
        end;


        define event StatErrorScheme;
            break / if ^any(CONTRASTCOLOR, COLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="Error" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
         style = GraphError;
         pure_style;
        end;

        define event StatBarScheme;
            break / if ^any(DISPLAYOPTS);
            ndent;
            put '<style:Style context="Bar" ';
            putq " displayOpts=" DISPLAYOPTS;
            put "/>" NL;
            xdent;
         style = GraphBar;
         pure_style;
        end;

        define event StatBoxMedianScheme;
            break / if ^any(CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                                LINETHICKNESS);
            ndent;
            put '<style:Style context="BoxMedian" ';

            trigger StatStyleColAltFillTextColors;
            trigger StatStyleLineAttrs;
            put "/>" NL;
            xdent;
         style = GraphBoxMedian;
         pure_style;
        end;


        define event StatBoxMeanScheme;
            break / if ^any(CONTRASTCOLOR, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="BoxMean" ';

            trigger StatStyleColAltFillTextColors;
            trigger StatStyleMarkerAttrs;
            put "/>" NL;
            xdent;
         style = GraphBoxMean;
         pure_style;
        end;

        define event StatBoxWhiskerScheme;
            break / if ^any(CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS);
            ndent;
            put '<style:Style context="BoxWhisker" ';

                trigger StatStyleColAltFillTextColors;
            trigger StatStyleLineAttrs;
            put "/>" NL;
            xdent;
         style = GraphBoxWhisker;
         pure_style;
        end;

        define event StatBoxScheme;
            break / if ^any(DISPLAYOPTS, CONNECT );
            ndent;
            put '<style:Style context="Box" ';
            putq " displayOpts=" DISPLAYOPTS;
            putq " connect=" CONNECT;
            putq " capStyle=" CAPSTYLE;
            put "/>" NL;
            xdent;
         style = GraphBox;
         pure_style;
        end;

         define event StatHistogramScheme;
            break / if ^any(DISPLAYOPTS);
            ndent;
            put '<style:Style context="Histogram" ';
            putq " displayOpts=" DISPLAYOPTS;
            put "/>" NL;
            xdent;
         style = GraphHistogram;
         pure_style;
        end;

         define event StatBandScheme;
            break / if ^any(DISPLAYOPTS);
            ndent;
            put '<style:Style context="Band" ';
            putq " displayOpts=" DISPLAYOPTS;
            put "/>" NL;
            xdent;
         style = GraphBand;
         pure_style;
        end;

         define event StatContourScheme;
            break / if ^any(DISPLAYOPTS);
            ndent;
            put '<style:Style context="Contour" ';
            putq " displayOpts=" DISPLAYOPTS;
            trigger StatStyleCRampAttrs;
            put "/>" NL;
            xdent;
         style = GraphContour;
         pure_style;
        end;

         define event StatEllipseScheme;
            break / if ^any(DISPLAYOPTS);
            ndent;
            put '<style:Style context="Ellipse" ';
            putq " displayOpts=" DISPLAYOPTS;
            put "/>" NL;
            xdent;
         style = GraphEllipse;
         pure_style;
        end;

        define event StatAnnoLineScheme;
            break / if ^any(CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                                LINETHICKNESS);
            ndent;
            put '<style:Style context="AnnoLine" ';

            trigger StatStyleColAltFillTextColors;
            trigger StatStyleLineAttrs;
            put "/>" NL;
            xdent;
         style = GraphAnnoLine;
         pure_style;
        end;

        define event StatAnnoTextScheme;
            style = GraphAnnoText;
            pure_style;
            break / if ^any(COLOR, FONT_FACE, FONT_STYLE, FONT_WEIGHT,
                            FONT_SIZE);
            ndent;
            put '<style:Style context="AnnoText"';

            trigger StatStyleTextAttrs;

            put "/>" NL;
            xdent;
        end;

        define event StatAnnoShapeScheme;
            break / if ^any(COLOR, CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="AnnoShape" ';

            trigger StatStyleColorFillLineMarkerAttrs;

           put "/>" NL;
            xdent;
         style = GraphAnnoShape;
         pure_style;
        end;

        define event StatSelectionScheme;
            break / if ^any(COLOR, CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="Selection" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
         style = GraphSelection;
         pure_style;
        end;

        define event StatConnectLineScheme;
            break / if ^any(CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                                    LINETHICKNESS);
            ndent;
            put '<style:Style context="ConnectLine" ';

            trigger StatStyleLineAttrs;
            put "/>" NL;
            xdent;
         style = GraphConnectLine;
         pure_style;
        end;

        define event StatMissingScheme;
            break / if ^any(COLOR, CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="Missing" ';

            trigger StatStyleColorFillLineMarkerAttrs;

            put "/>" NL;
            xdent;
         style = GraphMissing;
         pure_style;
        end;

        define event StatControlLimitsScheme;
            break / if ^any(COLOR, CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="ControlLimits" ';

            trigger StatStyleColorFillLineMarkerAttrs;

           put "/>" NL;
            xdent;
         style = GraphControlLimits;
         pure_style;
        end;

        define event StatRunTestScheme;
            break / if ^any(COLOR, CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                            LINETHICKNESS, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="RunTest" ';

            trigger StatStyleColorFillLineMarkerAttrs;

           put "/>" NL;
            xdent;
         style = GraphRunTest;
         pure_style;
        end;

        define event StatStarsScheme;
            break / if ^any(COLOR, CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                                        LINETHICKNESS);
            ndent;
            put '<style:Style context="Stars" ';

            trigger StatStyleFillAttrs;
            trigger StatStyleLineAttrs;
            put "/>" NL;
            xdent;
         style = GraphStars;
         pure_style;
        end;

        define event StatClippingScheme;
            break / if ^any(COLOR, CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH,
                             LINETHICKNESS, MARKERSYMBOL, MARKERSIZE);
            ndent;
            put '<style:Style context="Clipping" ';

            trigger StatStyleFillAttrs;
            trigger StatStyleLineAttrs;
            trigger StatStyleMarkerAttrs;
            put "/>" NL;
            xdent;
         style = GraphClipping;
         pure_style;
        end;

        define event StatBlockScheme;
            break / if ^any(COLOR);
            ndent;
            put '<style:Style context="Block"';
            putq " color=" COLOR;
            putq " fillColor=" COLOR;
            put "/>" NL;
            xdent;
         style = GraphBlock;
         pure_style;
        end;

        define event StatAltBlockScheme;
            break / if ^any(COLOR);
            ndent;
            put '<style:Style context="AltBlock"';
            putq " color=" COLOR;
            putq " fillColor=" COLOR;
            put "/>" NL;
            xdent;
         style = GraphAltBlock;
         pure_style;
        end;

/***********************************************************************/
/*          The end of the graph appearance scheme events.             */
/***********************************************************************/

        define event StatPlotError;
                /* Clear out all role, tipFormat, tipLabel and tipRole lists */
            unset $Roles;
            unset $TipFormats;
            unset $TipLabels;
            unset $TipRoleStr;
            unset $SortOrders;
                /* Log it as a warning. The extra " " works around a bug.
                 * (Eric Gebhart: it confuses the look ahead for the connection
                 * between labels and variables.)
                 * ReqArgsMissingMsg will only be set for SGRENDER, data null */
            putlog "%2z" " " @ReqArgsMissingMsg / if @ReqArgsMissingMsg;
        end;

        /* Used to be event StatGraph - we now do this from  BeginGraph
         * See StatLayout. $models is now unset in StatGraphView
         */
        define event StatGraphAttrs;

            putq " ID=" @graphid;

            putq " designHeight=" @DesignHeight;
            putq " designWidth=" @DesignWidth;

            putq " dataSizeScalingFactor=" @ScalingFactor;
            putq " gapScalingFactor=" @GapScalingFactor;
            putq " dataDPIScalingFactor=" @DPIScalingFactor;

            put ' style:style="StatGraphStyle';
            put anchor '"';

            putq " antiAlias=" @AntiAlias;
            putq " outmostLayoutBorder=" @OutmostLayoutBorder;
            putq " imagemapLimit=" @ImagemapLimit;
            putq " labelingLimit=" @LabelingLimit;
            putq " antiAliasLimit=" @AntiAliasLimit;
            putq " maxLegendArea=" @MaxLegendArea;
            putq " groupMax=" @GroupMax;
            putq " discreteMax=" @DiscreteMax;
            putq " panelCellMax=" @PanelCellMax;
            putq " sessionLocale=" @SessionLocale;
            putq " missingOption=" @MissingOption;
            putq " notesOption=" @NotesOption;
            putq " debugOption=" @DebugOption;
            putq " suppressClientMsg=" @SuppressClientMsg;
            putq " includeMissingDiscrete=" @IncludeMissingDiscrete;
            putq " scaleMarkers=" @ScaleMarkers;
            putq " collation=" @Collation;

            put ' version=' &_stg_version_;
        end;


        /**************
         * Attribute triggers.
         *************/

        define event StatComponentAttrs;
            putq " _id=" @_Id;
            putq " name=" @Name;
            putq " orient=" @Orient;
            putq " hAlign=" @HAlign;
            putq " vAlign=" @VAlign;
            putq " hAnchor=" @HAnchor;
            putq " vAnchor=" @VAnchor;
            putq " location=" @Location;
            putq " autoAlign=" @AutoAlign;
            trigger StatPadAttrs;
            putq " backgroundAttrs=" @BackgroundAttrs;
            putq " backgroundColor=" @BackgroundColor;
            putq " border=" @Border;
            putq " borderAttrs=" @BorderAttrs;
            putq " discreteOffset=" @DiscreteOffset;
            putq " displayClipped=" @DisplayClipped;
            putq " opaque=" @Opaque;
            putq " wallDisplay=" @WallDisplay;
            putq " wallColor=" @WallColor;
        end;

        define event StatPadAttrs;
            do / if any(@Left, @Right, @Top, @Bottom);
                ndent;
                put "/>" CR;
                xdent;
            done;
            putq " pad=" @Pad / if ^any(@Left, @Right, @Top, @Bottom);
            putq " outerpad=" @OuterPad / if ^any(@Left, @Right, @Top, @Bottom);
        end;

        define event StatRoleItem;
            /* This is a tagset list! - gets created implicitly */
            set $Roles[name] value;
        end;

        define event StatRoleList;
            break / if ^exists($Roles);
            ndent;
            put "<Roles>" CR;
            ndent;
            iterate $Roles;
            do / while _value_;
                putq "<Role name=" _name_ ">";
                put $models[_value_] anchor "#" _value_;
                put "</Role>" CR;
                next $Roles;
            done;
            xdent;
            put "</Roles>" CR;
            xdent;
            unset $Roles;
        end;

        define event StatTipFormatItem;
            /* This is a tagset list! - gets created implicitly */
            set $TipFormats[name] value;
        end;

        define event StatTipFormatList;
            break / if ^exists($TipFormats);
            ndent;
            put "<TipFormats>" CR;
            ndent;
            iterate $TipFormats;
            do / while _value_;
                putq "<TipFormat name=" _name_ ">";
                put _value_;
                put "</TipFormat>" CR;
                next $TipFormats;
            done;
            xdent;
            put "</TipFormats>" CR;
            xdent;
            unset $TipFormats;
        end;

        define event StatTipLabelItem;
            /* This is a tagset list! - gets created implicitly */
            set $TipLabels[name] value;
        end;

        define event StatTipLabelList;
            break / if ^exists($TipLabels);
            ndent;
            put "<TipLabels>" CR;
            ndent;
            iterate $TipLabels;
            do / while _value_;
                putq "<TipLabel name=" _name_ ">";
                put _value_;
                put "</TipLabel>" CR;
                next $TipLabels;
            done;
            xdent;
            put "</TipLabels>" CR;
            xdent;
            unset $TipLabels;
        end;

        /* Kludge event to concat strings */
        define event StatTipRoleItem;
            do / if ^exists($TipRoleStr);
                set $TipRoleStr name;
            else;
                set $TipRoleStr $TipRoleStr ' ' name;
            done;
        end;

        define event StatOutlierTipRoleItem;
            do / if ^exists($OutlierTipRoleStr);
                set $OutlierTipRoleStr name;
            else;
                set $OutlierTipRoleStr $OutlierTipRoleStr ' ' name;
            done;
        end;

        define event StatRoleTipFormatLabelLists;
            trigger StatRoleList;
            /* For now, ^^ we use role list here since it is always
             * togther with Tip format and labels.
             */
            trigger StatTipFormatList;
            trigger StatTipLabelList;
        end;

        define event StatSortOrderItem;
            /* This is a tagset list! - gets created implicitly */
            set $SortOrders[name] value;
        end;

        define event StatSortOrderList;
            break / if ^exists($SortOrders);
            ndent;
            put "<SortOrders>" CR;
            ndent;
            iterate $SortOrders;
            do / while _name_;
                putq "<SortOrder name=" _name_ ">";
                put _value_;
                put "</SortOrder>" CR;
                next $SortOrders;
            done;
            xdent;
            put "</SortOrders>" CR;
            xdent;
            unset $SortOrders;
        end;

        define event StatCommonAttrBundles;
            putq " fillAttrs=" @FillAttrs;
            putq " lineAttrs=" @LineAttrs;
            putq " markerAttrs=" @MarkerAttrs;
            putq " outlineAttrs=" @OutlineAttrs;
        end;

        define event StatCommonChartAttrs;
        start:
            trigger StatComponentAttrs;

            putq " _stmt=" @_Stmt;

            putq " baselineAttrs=" @BaselineAttrs;
            putq " baselineIntercept=" @BaselineIntercept;
            putq " barWidth=" @BarWidth;
            putq " categoryOrderRole=" @CategoryOrderRole;
            putq " clusterWidth=" @clusterWidth;
            putq " colorModel=" @ColorModel;
            putq " connectAttrs=" @ConnectAttrs;
            putq " clip=" @Clip;
            putq " display=" @Display;
            putq " dataSelectable=" @DataSelectable;
            putq " dataSkin=" @DataSkin;
            putq " dataTransparency=" @DataTransparency;

            putq " groupDisplay=" @GroupDisplay;
            putq " groupOrder=" @GroupOrder;

                    /* Block and DropLine */
            putq " label=" @Label;
            put ' label="' $models[@LabelVar] "#" @LabelVar '"'
                                                / if exists(@LabelVar);
            putq " labelAttrs=" @LabelAttrs;
            putq " labelPosition=" @LabelPosition;

            putq " legendLabel=" @LegendLabel;
            putq " primary=" @Primary;
            putq " renderer=" @Renderer;

            putq " includeMissingGroup=" @includeMissingGroup
                                                    / if exists(@GroupVar);

                /* Moved most common attrs here */
            trigger StatCommonAttrBundles;
            putq " justify=" @Justify;

            putq " reverseColorModel=" @ReverseColorModel;

            putq " specLimitLower=" @SpecLimitLower;
            put ' specLimitLower="' $models[@SpecLimitLowerVar] anchor "#"
                                        @SpecLimitLowerVar '"'
                                            / if exists(@SpecLimitLowerVar);
            putq " specLimitUpper=" @SpecLimitUpper;
            put ' specLimitUpper="' $models[@SpecLimitUpperVar] anchor "#"
                                        @SpecLimitUpperVar '"'
                                            / if exists(@SpecLimitUpperVar);

            putq " tip=" $TipRoleStr;
            unset $TipRoleStr;

            put ' url="' $models[@UrlVar] anchor "#" @UrlVar '"' /
                                    if exists(@UrlVar);

            putq " xAxis=" @XAxis;
            putq " yAxis=" @YAxis;
        end;

        define event StatCommonLayoutAttrs;
            trigger StatComponentAttrs;
                /*
                 * Dont script zero dimension on root layout that were set
                 * when the dimensions were copied to StatGraph template.
                 */
            putq " height=" @Height / if ^cmp(@Height, "0");
            putq " width=" @Width / if ^cmp(@Width, "0");

            putq " aspectRatio=" @AspectRatio;
        end;

        define event StatCommon3DAttrs;
            /* zVar cant be included: contour needs it but doesnt share
             * these other attrs  */
            putq " cube=" @Cube;
            putq " rotate=" @Rotate;
            putq " tilt=" @Tilt;
            putq " zoom=" @Zoom;
        end;

        define event StatVariableAttrs;
            %putModelVar(@YVar, y);
            %putModelVar(@XVar, x);
            %putModelVar(@ZVar, z);

            %putModelVar(@ColorResponse, colorResponse);
            putq " colorResponseAttrVar=" @ColorResponseAttrVar;

            trigger StatGroupVars;
        end;

        define event StatGroupVars;
            %putModelVar(@GroupVar, group);
            %putModelVar(@GroupIndex, index);
            putq " groupAttrVar=" @GroupAttrVar;

            %putModelVar(@ColorGroup, colorGroup);
            putq " colorGroupAttrVar=" @ColorGroupAttrVar;

            %putModelVar(@LineColorGroup, lineColorGroup);
            %putModelVar(@LineColorGroupIndex, lineColorGroupIndex);

            %putModelVar(@LinePatternGroup, linePatternGroup);
            %putModelVar(@LinePatternGroupIndex, linePatternGroupIndex);

            %putModelVar(@MarkerColorGroup, markerColorGroup);
            %putModelVar(@MarkerColorGroupIndex, markerColorGroupIndex);

            %putModelVar(@MarkerSymbolGroup, markerSymbolGroup);
            %putModelVar(@MarkerSymbolGroupIndex, markerSymbolGroupIndex);
        end;

        define event StatSlopeConstVar;
            putq " slope=" @Slope;
            %putModelVar(@SlopeVar, slope);
        end;

        define event StatOriginConstVars;
            putq " xOrigin=" @xOrigin;
            %putModelVar(@XOriginVar, xOrigin);
            putq " yOrigin=" @yOrigin;
            %putModelVar(@YOriginVar, yOrigin);
        end;

         /* Used by StatClipPlotList below */
        define event StatClipPlotItem;
            set $ClipPlots[] name;
        end;

        define event StatClipPlotList;
            %StatListMacro(ClipPlotList, $ClipPlots);
        end;

         /* Used by StatViewportPlotList below */
        define event StatViewportPlotItem;
            set $ViewportPlots[] name;
        end;

        define event StatViewportPlotList;
            %StatListMacro(ViewportPlotList, $ViewportPlots);
        end;

        /* QC kludge */
        define event StatQCInternal;
            do / if @_Internal and ^ $ploterror;
                ndent;
                put @_Internal CR ;
                xdent;
            done;
        end;

        define event StatLayout;
          start:
            ndent;
            put "<" name ;
            trigger StatCommonLayoutAttrs;
            trigger StatGraphAttrs  / if cmp(name, "StatGraph");
            trigger StatCommon3DAttrs / if cmp(name, "LayoutOverlay3D");

            putq " equateType=" @EquateType;

            putq " rows=" @Rows;
            putq " columns=" @Columns;
            putq " order=" @Order;
            putq " columnGutter=" @Columngutter;
            putq " rowGutter=" @Rowgutter;
            putq " shrinkFonts=" @ShrinkFonts;

            putq " columnDataRange=" @ColumnDataRange;
            putq " rowDataRange=" @RowDataRange;
            putq " column2DataRange=" @Column2DataRange;
            putq " row2DataRange=" @Row2DataRange;

            putq " cycleAttrs=" @CycleAttrs;
            putq " skipEmptyCells=" @SkipEmptyCells;
            putq " ignoreCase=" @IgnoreCase;
            putq " trimLeading=" @TrimLeading;
            putq " drawSpace=" @DrawSpace;
            putq " type=" @CatType;
            putq " interval=" @Interval;
            putq " numDev=" @NumDev;
            putq " levels=" @Levels;
            putq " colorModel=" @ColorModel;
            putq " reverseColorModel=" @ReverseColorModel;
            putq " palette=" @Palette;

            do  / if cmp(name, "LayoutLattice");
                putq " rowWeights=" @RowWeights;
                putq " columnWeights=" @ColumnWeights;
                putq " columnAxesLabelAlign=" @ColumnAxesLabelAlign;
                putq " rowAxesLabelAlign=" @RowAxesLabelAlign;
            done;

            do / if cmp(name, "DrawPolygon")
                        or cmp(name, "DrawPolyline");
                putq " display=" @Display;
                putq " fillAttrs=" @FillAttrs;
                putq " layer=" @Layer;
                putq " lineAttrs=" @LineAttrs;
                putq " rotate=" @Rotate;
                putq " transparency=" @Transparency;
                putq " x=" @X;
                putq " y=" @Y;
                putq " xAxis=" @XAxis;
                putq " yAxis=" @YAxis;
                putq " xSpace=" @XSpace;
                putq " ySpace=" @YSpace;
            done;

            do  / if cmp(name, "LayoutDataPanel")
                        or cmp(name, "LayoutDataLattice");

                put ' rowVar="' $models[@RowVar] anchor "#" @RowVar '"'
                                                    / if exists(@RowVar);
                put ' columnVar="' $models[@ColumnVar] anchor "#" @ColumnVar '"'
                                                    / if exists( @ColumnVar);

                putq " cellHeightMin=" @CellHeightMin;
                putq " cellWidthMin=" @CellWidthMin;
                putq " columnHeaders=" @ColumnHeaders;
                putq " columnWeight=" @ColumnWeight;
                putq " extractedHeaderNameAttrs=" @ExtractedHeaderNameAttrs;
                putq " headerBackgroundColor=" @HeaderBackgroundColor;
                putq " headerOpaque=" @HeaderOpaque;
                putq " headerLabelDisplay=" @HeaderLabelDisplay;
                putq " headerLabelAttrs=" @HeaderLabelAttrs;
                putq " headerLabelLocation=" @HeaderLabelLocation;
                putq " includeMissingClass=" @IncludeMissingClass;
                putq " panelNumber=" @PanelNumber;
                putq " rowHeaders=" @RowHeaders;
                putq " rowWeight=" @RowWeight;
                putq " sparse=" @Sparse;
                putq " start=" @Start;

                /* Renderer will draw a full strip if just text is scripted! */
                trigger StatStripLabelVars
                                    / if any(@StripLabelStart, @StripLabelEnd);
                /* OK to script visual attrs - ignored by renderer if no
                 * StripLabel columns exist */
                putq " stripLabelFillAttrs=" @StripLabelFillAttrs;

                putq " verticalRowHeaders=" @VerticalRowHeaders;
            done;

            do  / if cmp(name, "LayoutGlobalLegend");
                putq " type=" @SubType;
                putq " gutter=" @Gutter;
                putq " legendTitlePosition=" @LegendTitlePosition;

                putq " title=" @Title;
                putq " titleAttrs=" @TitleAttrs;

                putq " weights=" @Weights;
            done;

            put ">" CR;

            do  / if cmp(name, "LayoutDataPanel")
                                    or cmp(name, "LayoutDataLattice");
                trigger StatDPClassVariables / if exists($DPClassVars);
                trigger StatColumnVariables / if exists($ColumnVars);
                trigger StatRowVariables / if exists($RowVars);
                trigger StatInsetVariables / if exists($InsetVars);
            done;

            do / if cmp(name, "LayoutDataLattice");
                trigger StatRoleList;
                trigger StatSortOrderList;
            done;

            do  / if cmp(name, "LayoutOverlayMapTemp")
                       or cmp(name, "LayoutOverlayMap");
                trigger StatClipPlotList / if exists($ClipPlots);
                do / if any($ViewportPlots, @XMin, @XMax, @YMin, @YMax);
                    put "<ViewportOpts";
                    putq " plots=" @Plots;
                    putq " xMin=" @XMin;
                    putq " xMax=" @XMax;
                    putq " yMin=" @YMin;
                    putq " yMax=" @YMax;
                    put ">" CR;
                    trigger StatViewportPlotList / if exists($ViewportPlots);
                    put "</ViewportOpts>";
                done;
            done;

            trigger StatQCInternal;

          finish:
            unset $DPClassVars;
            unset $ColumnVars;
            unset $RowVars;
            unset $InsetVars;
            put "</" name ">" CR;
            xdent;
        end;

        define event StatStripLabelVars;
            put ' stripLabelStart="'
                $models[@StripLabelStart] anchor "#" @StripLabelStart '"'
                                                / if exists(@StripLabelStart);
            put ' stripLabelEnd="'
                $models[@StripLabelEnd] anchor "#" @StripLabelEnd '"'
                                                / if exists(@StripLabelEnd);
            put ' stripLabelText="'
                $models[@StripLabelText] anchor "#" @StripLabelText '"'
                                                / if exists(@StripLabelText);
        end;

        /* For Layout Sidebar, RowHeader et al. */
        define event StatSubLayout;
          start:
            ndent;
            put "<" name;
            putq " align=" @Align;
            putq " spaceFill=" @SpaceFill;
            put ">" CR;
          finish:
            put "</" name ">" CR;
            xdent;
        end;

        define event StatCommonEntryAttrs;
            trigger StatComponentAttrs;
            putq " editable=" @Editable;
            putq " rotate=" @Rotate;
            putq " textAttrs=" @TextAttrs; /* for _RESET_ */
            putq " textFitPolicy=" @TextFitPolicy;

        end;

        define event StatEntryItem;
        start:
            ndent;
            put "<" @ItemTypeStr;
            trigger StatCommonEntryAttrs;
            put ">";
            put CR / if ^cmp(@ItemTypeStr, "EntryItem");
            put @textvalue;
        finish:
            put "</" @ItemTypeStr ">" CR;
            xdent;
        end;

        define event StatEntryItemUnicode;
        start:
            ndent;
            put "<EntryItem";
            trigger StatCommonEntryAttrs;
            put ">";
            unset $hexValue;
            do / if exists(@unicodesymbol);  /* Lookup symbol's hex value */
                trigger initUnicodeMap / if ^$unicodeMap;
                set $hexValue $unicodeMap[@textvalue];
            else;
                set $hexValue @textValue;
            done;
            put '&#x' $hexValue ';' / if $hexValue;
        finish:
            put "</EntryItem>" CR;
            xdent;
        end;

        define event StatEntry;
          start:
            ndent;
            put  "<" @EntryTypeStr;
            trigger StatCommonEntryAttrs;
            put ">" CR;
          finish:
            put  "</" @EntryTypeStr ">" CR;
            xdent;
        end;

        define event StatAttrBag;
            break / if $ploterror or ^exists(@ObjName);

                    /* TextAttrs can be "()" */
            break / if ^cmp(@ObjName, "TextAttrs") and
                        ^any(
                            @Color, @Size,              /* Common */
                            @Weight, @Style, @Family,   /* Font, Marker */
                            @Pattern, @Thickness,       /* Line */
                            @Transparency,              /* Fill */
                            @Symbol,                    /* Marker */
                            @Left, @Right, @Top, @Bottom /* Pad */
                         /* @Image, @Scale, @FitPosition    TextureAttrs */
                            );
            ndent;
            put   "<" @ObjName;

            putq " color=" @Color;
            putq " size=" @Size;

            putq " weight=" @Weight;
            putq " style=" @Style;
            putq " family=" @Family;
            do / if @Pattern;
              do / if cmp(@ObjName, "FillPatternAttrs");
                putq " pattern=" @Pattern;
              else;
                set $pattern @Pattern;
                put " pattern=";
                trigger StatLinePattern;
                unset $pattern;
              done;
            done;
            putq " thickness=" @Thickness;

            putq " transparency=" @Transparency;

            putq " symbol=" @Symbol;

            putq " left=" @Left;
            putq " right=" @Right;
            putq " top=" @Top;
            putq " bottom=" @Bottom;

/*
            putq " image=" @Image;
            putq " scale=" @Scale;
            putq " fitPosition=" @FitPosition;
*/

            put   '/>' CR;
            xdent;
        end;

/*
        define event StatBackgroundOpts;
          start:
            break / if $ploterror or ^exists(name);

            ndent;
            put   "<" name;

            putq " type=" @Type;
            put  ">" CR;

          finish:
            break / if $ploterror or ^exists(name);
            put   "</" name ">" CR;
            xdent;
        end;
*/

        define event StatGradientStepperOpts;
          start:
            ndent;
            put   "<" name;
            putq " startColor=" @StartColor;
            putq " endColor=" @EndColor;
            putq " stepCount=" @StepCount;
            putq " stepIndex=" @StepIndex;
            putl " />";
            xdent;
        end;

        define event StatInsetOpts;
          start:
            ndent;
            put "<InsetOpts";
            trigger StatComponentAttrs;
            putq " title=" @Title;
            putq " titleAttrs=" @TitleAttrs;
            putq " textAttrs=" @TextAttrs;
            put   '>' CR;
          finish:
            put "</InsetOpts>" CR;
            xdent;
        end;

        define event StatSpecLimitOpts;
          start:
            ndent;
            put   "<" name;
            putq " fillAttrs=" @TextAttrs;
            putq " legendLabel=" @LegendLabel;
            put   '>' CR;
          finish:
            put   "</" name ">" CR;
            xdent;
        end;

        define event StatCurveLabelAttrs;
            putq " curveLabel=" @CurveLabel;
            put ' curveLabel="'
                    $models[@CurveLabelVar] anchor "#" @CurveLabelVar '"'
                                                / if exists(@CurveLabelVar);
            putq " curveLabelUpper=" @CurveLabelUpper;
            put ' curveLabelUpper="'
                    $models[@CurveLabelUpperVar] anchor "#" @CurveLabelUpperVar '"'
                                            / if exists(@CurveLabelUpperVar);
            putq " curveLabelLower=" @CurveLabelLower;
            put ' curveLabelLower="'
                    $models[@CurveLabelLowerVar] anchor "#" @CurveLabelLowerVar '"'
                                            / if exists(@CurveLabelLowerVar);
           do / if any(@CurveLabel, @CurveLabelVar,
                                @CurveLabelUpper, @CurveLabelUpperVar,
                                @CurveLabelLower, @CurveLabelLowerVar);
               putq " curveLabelLocation=" @CurveLabelLocation;
               putq " curveLabelPosition=" @CurveLabelPosition;
               putq " curveLabelAttrs=" @CurveLabelAttrs;
           done;
        end;

        define event StatDataLabelVarAttrs;
            putq " dataLabelAttrs=" @DataLabelAttrs;
            putq " dataLabelPosition=" @DataLabelPosition;
            put ' dataLabel="'
                   $models[@DataLabelVar] anchor "#" @DataLabelVar '"'
                                                / if exists(@DataLabelVar);
        end;

        define event StatErrorAttrs;
            put ' errorUpper="'
                    $models[@ErrorUpperVar] anchor "#" @ErrorUpperVar '"'
                                                    / if exists(@ErrorUpperVar);
            put ' errorLower="'
                    $models[@ErrorLowerVar] anchor "#" @ErrorLowerVar '"'
                                                    / if exists(@ErrorLowerVar);
            putq " errorBarAttrs=" @ErrorBarAttrs;
        end;


        /* Only StatScatter calls this right now! */
        define event StatXYErrorAttrs;
            put ' xErrorUpper="'
                    $models[@XErrorUpperVar] anchor "#" @XErrorUpperVar '"'
                                                    / if exists(@XErrorUpperVar);
            put ' xErrorLower="'
                    $models[@XErrorLowerVar] anchor "#" @XErrorLowerVar '"'
                                                    / if exists(@XErrorLowerVar);
            put ' yErrorUpper="'
                    $models[@YErrorUpperVar] anchor "#" @YErrorUpperVar '"'
                                                    / if exists(@YErrorUpperVar);
            put ' yErrorLower="'
                    $models[@YErrorLowerVar] anchor "#" @YErrorLowerVar '"'
                                                    / if exists(@YErrorLowerVar);
            putq " errorBarAttrs=" @ErrorBarAttrs;
        end;

        define event StatAxis;
          start:
            ndent;
            set $axisName @Coord name;
            put '<' $axisName;

                /* Component attr, but we dont call the common component
                 * event here */
            putq " _id=" @_Id;

            do / if $LabelTextList;
               put ' label="';
               iterate $LabelTextList;
               do / while _value_;
                  put _value_;
                  next $LabelTextList;
               done;
               put '"';
               unset $LabelTextList;
            else;
               putq " label=" @Label;
            done;
            putq " altDisplay=" @AltDisplay;
            putq " altDisplaySecondary=" @AltDisplaySecondary;
            putq " display=" @Display;
            putq " displaySecondary=" @DisplaySecondary;
            putq " tickStyle=" @TickStyle;
            putq " offsetMin=" @OffsetMin;
            putq " offsetMax=" @OffsetMax;
            putq " labelAttrs=" @LabelAttrs;
            putq " labelFitPolicy=" @LabelFitPolicy;
            putq " labelPosition=" @LabelPosition;
            putq " tickValueAttrs=" @TickValueAttrs;
            putq " gridAttrs=" @GridAttrs;
            putq " gridDisplay=" @GridDisplay;
            putq " name=" @Name;
            putq " type=" @Type;
            putq " reverse=" @Reverse;
            putq " shortLabel=" @ShortLabel;
            put ">" CR;
            trigger StatAxisTypeOptions / if exists(@Type);
          finish:
            put '</' $axisName '>' CR;
            unset $axisName;
            xdent;
        end;

            /* Used by StatAxisTickValueList below */
        define event StatAxisTickValueItem;
            set $TickValues[] name;
        end;

        define event StatAxisTickValueList;
            %StatListMacro(TickValues, $TickValues);
        end;

            /* Used by StatAxisTickDisplayList below */
        define event StatAxisTickDisplayItem;
            set $TickDisplays[] name;
        end;

        define event StatAxisTickDisplayList;
            %StatListMacro(TickDisplays, $TickDisplays);
        end;

        /* LinearOpts, LogOpts, DiscreteOpts, TimeOpts */
        define event StatAxisTypeOpts;
          start:
            break / if ^exists(@ObjName);
            ndent;
            put   "<" @ObjName;
            putq " base=" @Base;
            putq " colorBands=" @ColorBands;
            putq " colorBandsAttrs=" @ColorBandsAttrs;
            putq " tickIntervalStyle=" @TickIntervalStyle;
            putq " integer=" @Integer;
            putq " interval=" @Interval;
            putq " minorTicks=" @MinorTicks;
            putq " origin=" @Origin;
            putq " sortOrder=" @SortOrder;
            putq " splitTickValue=" @SplitTickValue;
            putq " thresholdMin=" @ThresholdMin;
            putq " thresholdMax=" @ThresholdMax;
            putq " tickBoxDisplay=" @TickBoxDisplay;
            putq " tickType=" @TickType;
            putq " tickValueFitPolicy=" @TickValueFitPolicy;
            putq " tickValueFormat=" @TickValueFormat;
            putq " tickValuePriority=" @TickValuePriority;
            putq " tickValueRotation=" @TickValueRotation;
            putq " viewMin=" @ViewMin;
            putq " viewMax=" @ViewMax;

            putq " viewportMin=" @ViewportMin;
            putq " viewportMax=" @ViewportMax;

            put ">" CR;

            trigger StatAxisTickValueList;
            trigger StatAxisTickDisplayList;

            do / if cmp(@ObjName, "LinearOpts");
                trigger StatAxisTickValueFormatOpts;
                trigger StatAxisTickValueSequence;
            done;
          finish:
            put "</" @ObjName ">" CR;
            xdent;
         end;

        /* Event for TickValueFormat object */
        define event StatAxisTickValueFormatOpts;
            break / if ^any(@MaxWidth, @MaxDecimals,
                                    @PreferredDecimals, @ExtractScale);
            ndent;
            put "<TickValueFormatOpts" ;
            putq " maxWidth=" @MaxWidth;
            putq " maxDecimals=" @MaxDecimals;
            putq " preferredDecimals=" @PreferredDecimals;
            putq " extractScale=" @ExtractScale;
            put "/>" CR;
            xdent;
        end;

        define event StatAxisTickValueSequence;
            break / if ^exists(@Start, @End, @Increment);
            ndent;
            put "<TickValueSequence" ;
            putq " start=" @Start;
            putq " end=" @End;
            putq " increment=" @Increment;
            put "/>" CR;
            xdent;
        end;

        define event StatDataValues;
          start:
            set $databucket ref_id;
            open $databucket / if $databucket;
            ndent;
            put "<Vs>" CR;
            do / if $databucket;
               close;
               unset $databucket;
            done;
          finish:
            set $databucket ref_id;
            open $databucket / if $databucket;
            put "</Vs>" CR;
            xdent;
            do / if $databucket;
               close;
               unset $databucket;
            done;
        end;

        define event StatDataValue;
          start:
            set $databucket ref_id;
            open $databucket / if $databucket;
            ndent;
            put "<V";
            putq " M=" missing;
            putq " E=" @Empty;
            put ">";
            put value;
            do / if $databucket;
               close;
               unset $databucket;
            done;
          finish:
            set $databucket ref_id;
            open $databucket / if $databucket;
            put "</V>" CR;
            xdent;
            do / if $databucket;
               close;
               unset $databucket;
            done;
        end;


        define event StatLegendItem;
          start:
            do / if ^exists(@Name,@Type);
               set $missingReqArgs "true";
               break;
            done;
            ndent;
            put "<LegendItem";

            trigger StatComponentAttrs;
            putq " type=" @Type;

            putq " fillAttrs=" @FillAttrs;
            putq " label=" @Label;
            putq " labelAttrs=" @LabelAttrs;
            putq " lineAttrs=" @LineAttrs;
            putq " markerAttrs=" @MarkerAttrs;
            putq " outlineAttrs=" @OutlineAttrs;
            putq " text=" @Text;
            putq " textAttrs=" @TextAttrs;

            put ">" CR;

          finish:
            %missingReqArgsHandler;
            put "</LegendItem>" CR;
            xdent;
        end;

        define event StatLegendGraphNameItem;
            set $GraphNames[] name;
        end;

        define event StatLegendGraphNameList;
            %StatListMacro(GraphNames, $GraphNames);
        end;

        /* Used by StatLegendExcludeList below */
        define event StatLegendExcludeListItem;
        set $ExcludeList[] name;
        end;

        define event StatLegendExcludeList;
        %StatListMacro(ExcludeList, $ExcludeList);
        end;

        define event StatLegend;
          start:
            ndent;
            put "<" name;

            putq " axisName=" @AxisName;

            putq " type=" @SubType;

            putq " title=" @Title;
            putq " titleAttrs=" @TitleAttrs;
            putq " valueAttrs=" @ValueAttrs;
            trigger StatComponentAttrs;

            putq " across=" @Across;
            putq " down=" @Down;
            putq " order=" @Order;
            putq " merge=" @Merge;
            putq " autoItemSize=" @AutoItemSize;
            putq " sortOrder=" @SortOrder;
            putq " titleBorder=" @TitleBorder;

            putq " displayMissing=" @DisplayMissing;
            putq " valueCount=" @ValueCount;
            putq " valueCountHint=" @ValueCountHint;
            put ">" CR;
            trigger StatLegendGraphNameList / if ^exists(@AxisName);
            trigger StatLegendExcludeList / if exists($ExcludeList);

          finish:
            put "</" name ">" CR;
            xdent;
        end;

        define event StatDiagonal;
          start:
            ndent;
            put "<Diagonal>" CR;
          finish:
            put "</Diagonal>" CR;
            xdent;
        end;

        /* Make sure $MapIDVars gets unset after usage */
        /* The modelname is custom in this list generation */
        define event StatMapIDVariables;
           break / if ^exists($MapIDVars);
           ndent;
            put "<MapIDList>" CR;
           ndent;
           iterate $MapIDVars;
           do / while _value_;
               put "<Value>";
               put $mapModel anchor "#" _value_;
               put "</Value>" CR;
               next $MapIDVars;
           done;
           xdent;
           put "</MapIDList>" CR;
           xdent;
           unset $MapIDVars;
        end;

        define event StatMapIDVars;
          set $MapIDVars[] value;
          putlog "map ID vars";
        end;

        /* Make sure $MapIDVars gets unset after usage */
        /* The modelname is custom in this list generation */
        define event StatMapIDVariables2;
           break / if ^exists($MapIDVars2);
           ndent;
           put "<IDList>" CR;
           ndent;
           iterate $MapIDVars2;
           do / while _value_;
               put "<Value>";
               put $models[_value_] anchor "#" _value_;
               put "</Value>" CR;
               next $MapIDVars2;
           done;
           xdent;
           put "</IDList>" CR;
           xdent;
           unset $MapIDVars2;
        end;

        define event StatMapIDVars2;
          set $MapIDVars2[] value;
          putlog "ID vars";
        end;

        /* Make sure $DPClassVars gets unset after usage */
        define event StatDPClassVariables;
            %StatVariableListMacro(ClassVars, $DPClassVars);
        end;

        define event StatDPClassVars;
          set $DPClassVars[] value;
        end;

        /* Used by StatColumnVariables */
        define event StatColumnVars;
          set $ColumnVars[] value;
        end;

        /* Used by StatRowVariables */
        define event StatRowVars;
          set $RowVars[] value;
        end;

        define event StatInsetVars;
          set $InsetVars[] value;
        end;

        define event StatTileByVars;
          set $TileByVars[] value;
        end;

        define event StatLabelTextList;
          set $LabelTextList[] value;
        end;

        define event StatXBoundaryValueItem;
            set $XBoundaryValues[] name;
        end;

        define event StatXBoundaryValueList;
            %StatListMacro(XBoundaryValues, $XBoundaryValues);
        end;

        define event StatYBoundaryValueItem;
            set $YBoundaryValues[] name;
        end;

        define event StatYBoundaryValueList;
            %StatListMacro(YBoundaryValues, $YBoundaryValues);
        end;

       define event StatInitPreCycle;
         start:
           eval $color_cnt 0;
           eval $ccolor_cnt 0;
           eval $msymbol_cnt 0;
           eval $lstyle_cnt 0;
           eval $color_idx 1;
           eval $ccolor_idx 1;
           eval $msymbol_idx 1;
           eval $lstyle_idx 1;
           set $setcolor "true";
           set $setccolor "true";
           set $setmsymbol "true";
           set $setlstyle "true";
           set $autocycleattrs "true";
         finish:
           unset $color_cnt;
           unset $ccolor_cnt;
           unset $msymbol_cnt;
           unset $lstyle_cnt;
           unset $color_idx;
           unset $ccolor_idx;
           unset $msymbol_idx;
           unset $lstyle_idx;
           unset $setcolor;
           unset $setccolor;
           unset $setmsymbol;
           unset $setlstyle;
           unset $autocycleattrs;
       end;

       define event StatPreCycle;
          pure_style;
          do / if COLOR and $setcolor;
             eval $color_cnt $color_cnt+1;
          else;
             unset $setcolor;
          done;
          do / if CONTRASTCOLOR and $setccolor;
             eval $ccolor_cnt $ccolor_cnt+1;
          else;
             unset $setccolor;
          done;
          do / if MARKERSYMBOL and $setmsymbol;
             eval $msymbol_cnt $msymbol_cnt+1;
          else;
             unset $setmsymbol;
          done;
          do / if LINESTYLE and $setlstyle;
             eval $lstyle_cnt $lstyle_cnt+1;
          else;
             unset $setlstyle;
          done;
       end;

       define event StatPreCycleRollout;
          eval $tempcolor min($color_cnt, $ccolor_cnt);
          do / if $tempcolor = 0;
             eval $tempcolor max($color_cnt, $ccolor_cnt);
          done;
          eval $tempshape min($msymbol_cnt, $lstyle_cnt);
          do / if $tempshape = 0;
             eval $tempshape max($msymbol_cnt, $lstyle_cnt);
          done;
          do / if ($tempcolor > 0) and ($tempshape > 0);
             eval $color_cnt $tempcolor*$tempshape;
             eval $ccolor_cnt $tempcolor*$tempshape;
             eval $msymbol_cnt $tempcolor*$tempshape;
             eval $lstyle_cnt $tempcolor*$tempshape;
          done;
       end;


/***********************************************************************/
/*          StatGraph plots and charts                                 */
/***********************************************************************/

        define event StatAttrStmt;
          start:
            ndent;
            put "<Attr" name;
            putq " value=" @Value;
            putq " valueType=" @ValueType;
            putq " minValue=" @MinValue;
            put ' minExclude="true"' / if @MinExclude;
            putq " maxValue=" @MaxValue;
            put ' maxExclude="true"' / if @MaxExclude;
            putq " rangeAltColorModel=" @RangeAltColorModel;
            putq " rangeColorModel=" @RangeColorModel;

/* This is for v930 templates. We started using nested StatGradientStepperOpts
 * in v930M1. Keep these for backward compat. */
            do / if @StartColor;
                 put ' rangeColor="GradientStepper(';
                 put @StartColor ',';
                 put @EndColor ',';
                 put @StepCount ',';
                 put @StepIndex ')"';
            else;
                   putq " rangeColor=" @RangeColor;
            done;
            putq " rangeAltColor=" @RangeAltColor;
            trigger StatCommonChartAttrs;
            put ">" CR;
            do / if exists($autocycleattrs);
              iterate $AttrMapValues; /* There is only one value in this case */
              do / if cmp(" ", _value_) or cmp(".", _value_);
                 trigger StatAttrMapMissingAttrs;
              else;
                 trigger StatCycleAttrs;
              done;
            done;
            trigger StatAttrMapValueList;
            trigger StatRangeAltColorValueList;
            trigger StatRangeColorValueList;
          finish:
            put "</Attr" name ">" CR;
            xdent;
        end;

        define event StatAttrMapMissingAttrs;
            ndent;
            put '<FillAttrs color="Missing:color" />' CR;
            put '<LineAttrs';
            put ' color="Missing:altColor"';
            put ' pattern="Missing:lineStyle"';
            put ' />' CR;
            put '<MarkerAttrs';
            put ' color="Missing:altColor"';
            put ' symbol="Missing:markerSymbol"';
            put ' />' CR;
            xdent;
        end;


        define event StatCycleAttrs;
            ndent;
            put '<FillAttrs color="GraphDataStyle' $color_idx ':color" />' / if $color_idx > 0;
            put '<LineAttrs';
            put ' color="GraphDataStyle' $ccolor_idx ':altColor"' / if $ccolor_idx > 0;
            put ' pattern="GraphDataStyle' $lstyle_idx ':lineStyle"'  / if $lstyle_idx > 0;
            put ' />';
            put '<MarkerAttrs';
            put ' color="GraphDataStyle' $ccolor_idx ':altColor"' / if $ccolor_idx > 0;
            put ' symbol="GraphDataStyle' $msymbol_idx ':markerSymbol"' / if $msymbol_idx > 0;
            put ' />';
            do / if $color_idx = $color_cnt;
               eval $color_idx 1;
            else;
               eval $color_idx $color_idx+1;
            done;
            do / if $ccolor_idx = $ccolor_cnt;
               eval $ccolor_idx 1;
            else;
               eval $ccolor_idx $ccolor_idx+1;
            done;
            do / if $lstyle_idx = $lstyle_cnt;
               eval $lstyle_idx 1;
            else;
               eval $lstyle_idx $lstyle_idx+1;
            done;
            do / if $msymbol_idx = $msymbol_cnt;
               eval $msymbol_idx 1;
            else;
               eval $msymbol_idx $msymbol_idx+1;
            done;
            xdent;
        end;

        define event StatAttrMapValueItem;
            do / if cmp(value, "___EMPTY___");
               set $AttrMapValues[]  "___EMPTY___";
            else;
               set $AttrMapValues[] name;
            done;
        end;

        define event StatAttrMapValueList;
            %StatListMacro(Values, $AttrMapValues);
        end;

        define event StatAltColorValueItem;
             set $AltColorValues[] name;
        end;

        define event StatRangeAltColorValueList;
            %StatListMacro(AltColors, $AltColorValues);
        end;

        define event StatColorValueItem;
             set $ColorValues[] name;
        end;

        define event StatRangeColorValueList;
            %StatListMacro(Colors, $ColorValues);
        end;

        define event StatAttrVar;
          start:
            do / if ^exists(@AttrMap, @AttrVar);
                /* techinically only attrmap can be dynamic ... */
                set $missingReqArgs "true";
                break;
            done;
            ndent;
            put "<" name;
            putq " attrVar=" @AttrVar;
            /* @Variable unused - gets scripted directly on the plot */
            putq " attrMap=" @AttrMap;
            put ">" CR;
          finish:
            do / if $missingReqArgs;
                /* Probably needs a log message: see missingReqArgsHandler */
                unset $missingReqArgs;
                break;
            done;
            put "</" name ">" CR;
            xdent;
        end;

        define event StatDraw;
          start:
            ndent;
            put "<" name;
            putq " _id=" @_Id;  /* Overkill to use StatComponentAttrs */

            putq " anchor=" @Anchor;
            putq " arrowHeadDirection=" @ArrowHeadDirection;
            putq " arrowHeadShape=" @ArrowHeadShape;
            putq " arrowHeadScale=" @ArrowHeadScale;
            putq " backgroundAttrs=" @BackgroundAttrs;
            putq " border=" @Border;
            putq " borderAttrs=" @BorderAttrs;
            putq " cornerRadius=" @CornerRadius;
            putq " display=" @Display;

            putq " discreteOffset=" @DiscreteOffset;
                                    /* Overkill to use StatComponentAttrs */

            putq " fillAttrs=" @FillAttrs;
            putq " justify=" @Justify;
            putq " layer=" @Layer;
            putq " lineAttrs=" @LineAttrs;
            putq " rotate=" @Rotate;
            putq " transparency=" @Transparency;
            putq " width=" @Width;
            putq " height=" @Height;
            putq " widthUnit=" @WidthUnit;
            putq " heightUnit=" @HeightUnit;
            putq " sizeUnit=" @SizeUnit;
            putq " image=" @Image;
            putq " scale=" @Scale;
            putq " x=" @X;
            putq " y=" @Y;
            putq " x1=" @X1;
            putq " y1=" @Y1;
            putq " x2=" @X2;
            putq " y2=" @Y2;
            putq " xAxis=" @XAxis;
            putq " yAxis=" @YAxis;
            putq " x1Axis=" @X1Axis;
            putq " y1Axis=" @Y1Axis;
            putq " x2Axis=" @X2Axis;
            putq " y2Axis=" @Y2Axis;
            putq " xSpace=" @XSpace;
            putq " ySpace=" @YSpace;
            putq " x1Space=" @X1Space;
            putq " y1Space=" @Y1Space;
            putq " x2Space=" @X2Space;
            putq " y2Space=" @Y2Space;
            putq " drawSpace=" @DrawSpace;
            put ">" CR;
          finish:
            put "</" name ">" CR;
            xdent;
        end;

        define event StatMapChart;
          start:
            ndent;
            put "<Map";
            trigger StatCommonChartAttrs;
            put ">" CR;
          finish:
            put "</Map>" CR;
            xdent;
        end;

        define event StatMapRegionLayer;
          start:
            ndent;
            put "<MapRegionLayer";
            put ' mapProvider="SASMapData"';
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;
            put ' ID="' $models[@IDVar] anchor '#' @IDVar '"'/ if @IDVar;
            putq " mapID=" @MapIDVar;
            putq " emptyFillAttrs=" @EmptyFillAttrs;
            putq " emptyOutlineAttrs=" @EmptyOutlineAttrs;
            putq " includeMissingColor=" @IncludeMissingColor;
            putq " missingFillAttrs=" @MissingFillAttrs;
            putq " stat=" @Stat;
            put ">" CR;
          finish:
            put "</MapRegionLayer>" CR;
            xdent;
        end;

        define event StatStep;
          start:
            do / if ^exists(@XVar,@YVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<StepPlot";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;
            trigger StatDataLabelVarAttrs;
            trigger StatErrorAttrs;

            putq " connectOrder=" @ConnectOrder;
            putq " break=" @Break;
            putq " join=" @Join;
            trigger StatCurveLabelAttrs;
            put ">" CR;
            trigger StatRoleTipFormatLabelLists;
          finish:
            do / if ^exists(@XVar,@YVar);
                 unset $ploterror;
                trigger StatPlotError;
                 break;
            done;
            put "</StepPlot>" CR;
            xdent;
        end;

        define event StatMarkerCharacterAttrs;
            break / if ^exists(@MarkerCharacter);
            put ' markerCharacter="' $models[@MarkerCharacter]
                    anchor "#" @MarkerCharacter '"';
            putq " markerCharacterAttrs=" @MarkerCharacterAttrs;
         end;


        define event StatFreqVar;
            put ' freq="' $models[@FreqVar]
                    anchor "#" @FreqVar '"' / if exists(@FreqVar);
        end;


        define event StatMarkerColorGradientVar;
            put ' markerColorGradient="' $models[@MarkerColorGradient]
                        anchor "#" @MarkerColorGradient '"'
                                                / if exists(@MarkerColorGradient);
            putq " markerColorGradientAttrVar=" @MarkerColorGradientAttrVar;
        end;

        define event StatScatter;
          start:
            do / if ^exists(@XVar,@YVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<ScatterPlot";
            trigger StatVariableAttrs;
            trigger StatMarkerColorGradientVar;
            trigger StatMarkerCharacterAttrs;
            trigger StatCommonChartAttrs;
            trigger StatDataLabelVarAttrs;
            trigger StatXYErrorAttrs;
            trigger StatFreqVar;

            putq " discreteMarkerSize=" @DiscreteMarkerSize;

            %putModelVar(@MarkerSizeResponseVar, markerSizeResponse);
            putq " markerSizeMin=" @MarkerSizeMin;
            putq " markerSizeMax=" @MarkerSizeMax;

            putq " useDiscreteSize=" @UseDiscreteSize;

            put ">" CR;

            trigger StatRoleTipFormatLabelLists;

          finish:
            %plotErrorFinish;
            put "</ScatterPlot>" CR;
            xdent;
        end;


        /* Called by SCMX and DataLattice.
         * Make sure $ColumnVars gets unset after usage */
        define event StatColumnVariables;
            %StatVariableListMacro(ColumnVars, $ColumnVars);
        end;

        /* Called by SCMX and DataLattice.
         * Make sure $RowVars gets unset after usage */
        define event StatRowVariables;
            %StatVariableListMacro(RowVars, $RowVars);
        end;

        /* Make sure $TileByVars gets unset after usage */
        define event StatTileByVariables;
            %StatVariableListMacro(TileBy, $TileByVars);
        end;

        /* Make sure $InsetVars gets unset after usage */
        define event StatInsetVariables;
            %StatVariableListMacro(InsetVars, $InsetVars);
        end;

        define event StatScatterPlotMatrix;
          start:
            do / if ^exists($ColumnVars);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<ScatterPlotMatrix";
            trigger StatGroupVars;
            trigger StatMarkerColorGradientVar;
            putq " start=" @Start;

            trigger StatMarkerCharacterAttrs;
            trigger StatCommonChartAttrs;
            trigger StatDataLabelVarAttrs;
            trigger StatFreqVar;

            put ">" CR;

            trigger StatColumnVariables;
            trigger StatRowVariables / if exists($RowVars);
            trigger StatInsetVariables / if exists($InsetVars);
            trigger StatRoleTipFormatLabelLists;

          finish:
            %plotErrorFinish;
            put "</ScatterPlotMatrix>" CR;
            xdent;
            unset $ColumnVars;
            unset $RowVars;
            unset $InsetVars;
        end;


        define event StatSGanno;
          start:
            ndent;
            put  "<SGanno";
            put ' subsetVar="' $annotateModel anchor "#" @SubsetVar '"'
                                                    / if exists(@SubsetVar);
            putq " subsetValue=" @SubsetValue;
            putq " scope=" @Scope;
            put ' model="' $annotateModel anchor '"' / if $annotateModel;
            put ">" CR;
          finish:
            put  "</SGanno>" CR;
            xdent;
        end;


        define event StatHighLowValid;
          unset $validHighLow;
          set $validHighLow "1" / if any(@Xvar, @Yvar) and
              exist(@HighVar) and exists(@LowVar);
        end;

        define event StatHighLow;
        start:

        trigger StatHighLowValid;
          do / if ^exists($validHighLow);
          set $ploterror "true";
          break;
          done;

          ndent;
          put "<HighLowPlot";
          trigger StatVariableAttrs;
          trigger StatCommonChartAttrs;

          put ' high="'$models[@HighVar] anchor "#" @HighVar '"' / if exists(@HighVar);
          put ' low="'$models[@LowVar] anchor "#" @LowVar '"' / if exists(@LowVar);
          put ' open="'$models[@OpenVar] anchor "#" @OpenVar '"' / if exists(@OpenVar);
          put ' close="'$models[@CloseVar] anchor "#" @CloseVar '"' / if exists(@CloseVar);
          put ' lowCap="'$models[@LowCapVar] anchor "#" @LowCapVar '"' / if exists(@LowCapVar);
          put ' highCap="'$models[@HighCapVar] anchor "#" @HighCapVar '"' / if exists(@HighCapVar);
          put ' lowLabel="'$models[@LowLabelVar] anchor "#" @LowLabelVar '"' / if exists(@LowLabelVar);
          put ' highLabel="'$models[@HighLabelVar] anchor "#" @HighLabelVar '"' / if exists(@HighLabelVar);
          putq " lowCap=" @LowCap;
          putq " highCap=" @HighCap;
          putq " intervalBarWidth=" @IntervalBarWidth;
          putq " type=" @Type;
          put ">" CR;
          trigger StatRoleTipFormatLabelLists;

        finish:
          do / if ^exists($validHighLow);
          unset $ploterror;
          trigger StatPlotError;
          break;
          done;
          put "</HighLowPlot>" CR;
          xdent;
        end;


        define event StatBandValid;
            unset $validBand;
            set $validBand "1" / if any(@Xvar, @Yvar) and
                                        any(@LimitUpper, @LimitUpperVar) and
                                            any(@LimitLower, @LimitLowerVar);
        end;

        define event StatBand;
          start:

            trigger StatBandValid;
            do / if ^exists($validBand);
               set $ploterror "true";
               break;
            done;

            ndent;
            put "<BandPlot";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;

            putq " limitUpper=" @LimitUpper;
            put ' limitUpper="'
                    $models[@LimitUpperVar] anchor "#" @LimitUpperVar '"'
                                                    / if exists(@LimitUpperVar);
            putq " limitLower=" @LimitLower;
            put ' limitLower="'
                    $models[@LimitLowerVar] anchor "#" @LimitLowerVar '"'
                                                    / if exists(@LimitLowerVar);
            trigger StatCurveLabelAttrs;
            putq " connectOrder=" @ConnectOrder;
            putq " extend=" @Extend;
            putq " modelName=" @ModelName;
            putq " type=" @Type;
            put ">" CR;
            trigger StatRoleTipFormatLabelLists;

          finish:
            %ploterrorfinish;
            put "</BandPlot>" CR;
            xdent;
        end;

        define event StatQCLimits;
          start:
            do / if ^( exists(@XVar) and
                        any(@LCL, @LCLVar, @CTL, @CTLVar, @UCL, @UCLVar));
               set $ploterror "true";
               break;
            done;

            ndent;
            put "<QCLimitsPlot";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;

            putq " lcl=" @LCL;
            %putModelVar(@LCLVar, lcl);

            putq " ctl=" @CTL;
            %putModelVar(@CTLVar, ctl);

            putq " ucl=" @UCL;
            %putModelVar(@UCLVar, ucl);

            putq " lclLabelAttrs=" @LCLLabelAttrs;
            putq " ctlLabelAttrs=" @CTLLabelAttrs;
            putq " uclLabelAttrs=" @UCLLabelAttrs;

            putq " lclLineAttrs=" @LCLLineAttrs;
            putq " ctlLineAttrs=" @CTLLineAttrs;
            putq " uclLineAttrs=" @UCLLineAttrs;

            putq " sigmaLabelAttrs=" @SigmaLabelAttrs;

            putq " zoneAFillAttrs=" @ZoneAFillAttrs;
            putq " zoneBFillAttrs=" @ZoneBFillAttrs;
            putq " zoneCFillAttrs=" @ZoneCFillAttrs;

            putq " zoneAOutlineAttrs=" @ZoneAOutlineAttrs;
            putq " zoneBOutlineAttrs=" @ZoneBOutlineAttrs;
            putq " zoneCOutlineAttrs=" @ZoneCOutlineAttrs;

            putq " zoneLabelAttrs=" @ZoneLabelAttrs;

            put ">" CR;

            trigger StatQCInternal;

          finish:
            %plotErrorFinish;
            put "</QCLimitsPlot>" CR;
            xdent;
        end;

        define event StatBar;
          start:
            do / if (name eq "AreaBarParm");
                set $ploterror "true"
                    / if ^exists(@ResponseHeightVar, @ResponseWidthVar);
            else;
                /* Temp kludge for BarChart which can have just X */
               set $ploterror "true"
                    / if (^exists(@XVar)) or (length(@_Stmt) > 8
                                                        and ^exists(@YVar));
            done;
            break / if $ploterror;

            ndent;
            put "<" name;
            trigger StatVariableAttrs;

            %putModelVar(@ResponseHeightVar, responseHeight);
            %putModelVar(@ResponseWidthVar, responseWidth);
            %putModelVar(@Target, target);

            trigger StatDataLabelVarAttrs;
            trigger StatCommonChartAttrs;

            putq " barLabel=" @BarLabel;
            putq " barLabelFormat=" @BarLabelFormat;

            putq " connectColor=" @ConnectColor;
            putq " connectThickness=" @ConnectThickness;

            putq " dataLabelOrientation=" @DataLabelOrientation;

            putq " fillPatternAttrs=" @FillPatternAttrs;

            putq " skin=" @Skin;
            putq " stat=" @Stat;

            trigger StatErrorAttrs;

            put ">" CR;
            trigger StatRoleTipFormatLabelLists;
          finish:
            %plotErrorFinish;
            put "</" name ">" CR;
            xdent;
        end;

        define event StatWaterfall;
          start:
            do / if ^exists(@Category)  or ^exists(@Response);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<WaterfallChart";
            put ' category="' $models[@Category] anchor "#" @Category '"'
                                                    / if exists(@Category);
            put ' response="' $models[@Response] anchor "#" @Response '"'
                                                    / if exists(@Response);
            trigger StatVariableAttrs;
            trigger StatDataLabelVarAttrs;
            trigger StatCommonChartAttrs;
            putq " barLabel=" @BarLabel;
            putq " barLabelAttrs=" @BarLabelAttrs;
            putq " barLabelFormat=" @BarLabelFormat;
            putq " initialBarValue=" @InitialBarValue;
            putq " initialBarTickValue=" @InitialBarTickValue;
            putq " initialBarAttrs=" @InitialBarAttrs;
            putq " fillType=" @FillType;
            putq " finalBarTickValue=" @FinalBarTickValue;
            putq " finalBarAttrs=" @FinalBarAttrs;
            putq " stat=" @Stat;
            putq " connect=" @Connect;
            putq " connectDecreasingAttrs=" @ConnectDecreasingAttrs;
            putq " connectIncreasingAttrs=" @ConnectIncreasingAttrs;
            put ">" CR;
            trigger StatRoleTipFormatLabelLists;
          finish:
            %plotErrorFinish;
            put "</WaterfallChart>" CR;
            xdent;
        end;


        define event StatCommonHistogramAttrs;
            putq " xValues=" @XValues;
            putq " binAxis=" @BinAxis;
            putq " endLabels=" @EndLabels;
        end;

        define event StatHistogram;
          start:
            do / if ^exists(@XVar,@YVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<HistogramParm";
            trigger StatVariableAttrs;
            trigger StatDataLabelVarAttrs;
            trigger StatCommonChartAttrs;
            trigger StatCommonHistogramAttrs;
            putq " scale=" @scale;
            put ">" CR;
            trigger StatRoleTipFormatLabelLists;
          finish:
            do / if ^exists(@XVar,@YVar);
                 unset $ploterror;
                trigger StatPlotError;
                 break;
            done;
            put "</HistogramParm>" CR;
            xdent;
        end;

        define event StatBVHistogram;
          start:
            do / if ^exists(@XVar,@YVar, @ZVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<BiHistogram3DParm";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;
            trigger StatCommon3DAttrs;
            trigger StatCommonHistogramAttrs;
            putq " yValues=" @YValues;
            put ">" CR;
          finish:
            do / if ^exists(@XVar, @YVar, @ZVar);
                 unset $ploterror;
                trigger StatPlotError;
                 break;
            done;
            put "</BiHistogram3DParm>" CR;
            xdent;
        end;

        define event StatHeatmapParm;
          start:
            do / if ^exists(@XVar,@YVar) or ^any(@ColorGroup,@ColorResponse);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<HeatmapParm";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;
            putq " xValues=" @XValues;
            putq " yValues=" @YValues;
            putq " xGap=" @XGap;
            putq " yGap=" @YGap;
            putq " xBinAxis=" @XBinAxis;
            putq " yBinAxis=" @YBinAxis;
            putq " xEndLabels=" @XEndLabels;
            putq " yEndLabels=" @YEndLabels;
            put ">" CR;
            trigger StatXBoundaryValueList;
            trigger StatYBoundaryValueList;
            trigger StatRoleTipFormatLabelLists;
          finish:
            %plotErrorFinish;
            put "</HeatmapParm>" CR;
            xdent;
        end;

        define event StatKPITypeOpts;
            break / if $ploterror;
            break / if ^exists(@ObjName);

            ndent;
            put   "<" @ObjName;

            putq " orient=" @Orient;
            putq " majorTicks=" @MajorTicks;
            putq " minorTicks=" @MinorTicks;
            putq " type=" @Type;

            put   '/>' CR;
            xdent;
        end;

            /* Used by StatKPIBoundsList below */
        define event StatKPIBoundsItem;
            set $KPIBounds[name] type;  /* set as asc array: will this
                                         * preserve order ?? */
        end;

            /* Used by StatKPI below */
        define event StatKPIBoundsList;
            %StatTypedListMacro(Bounds, $KPIBounds);
        end;

            /* Used by StatColorsList below */
        define event StatColorsItem;
            set $Colors[] name;  /* set as list */
        end;

        define event StatColorsList;
            %StatListMacro(Colors, $Colors);
        end;

            /* Used by StatActiveColorsList below */
        define event StatActiveColorsItem;
            set $ActiveColors[] name;  /* set as list */
        end;

        define event StatActiveColorsList;
            %StatListMacro(ActiveColors, $ActiveColors);
        end;

        define event StatKPI;
          start:
            do / if ^(((exists(@Value) and ^cmp(@Value, '.'))
                                    or exists(@ValueVar)) and $KPIBounds);
                             /* Hopefully this resolves to exists($KPIBounds) */
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<KPIParm";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;

            putq " value=" @Value;
            %putModelVar(@ValueVar, value);

            putq " bounds=" @Bounds;

            putq " activeColors=" @ActiveColors;
            putq " colors=" @Colors;
            putq " kpiSkin=" @KpiSkin;
            putq " rangeLabelAttrs=" @RangeLabelAttrs;
            putq " rangeLabelFormat=" @RangeLabelFormat;

            putq " stat=" @Stat;
            putq " target=" @Target;
            %putModelVar(@TargetVar, target);

            putq " type=" @Type;
            putq " valueLabelAttrs=" @ValueLabelAttrs;
            putq " valueLabelFormat=" @ValueLabelFormat;
            put ">" CR;

            trigger StatKPIBoundsList;
            trigger StatColorsList;
            trigger StatActiveColorsList;

          finish:
            %plotErrorFinish;
            put "</KPIParm>" CR;
            xdent;
        end;

        define event StatBubblePlot;
          start:
            do / if ^exists(@XVar,@YVar, @SizeVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<BubblePlot";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;

            %putModelVar(@SizeVar, size);

            putq " bubbleRadiusMin=" @BubbleRadiusMin;
            putq " bubbleRadiusMax=" @BubbleRadiusMax;
            putq " relativeScale=" @RelativeScale;
            trigger StatDataLabelVarAttrs;
            put ">" CR;
            trigger StatRoleTipFormatLabelLists;
          finish:
            %plotErrorFinish;
            put "</BubblePlot>" CR;
            xdent;
        end;

      define event StatTreemap;
          start:
            do / if ^exists($TileByVars);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<TreeMap";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;

            %putModelVar(@SizeVar, size);

            putq " dataLabelLevel=" @dataLabelLevel;
            putq " detailLevel=" @DetailLevel;
            putq " arrangement=" @Arrangement;
            putq " statusTextAttrs=" @StatusTextAttrs;
            trigger StatDataLabelVarAttrs;
            put ">" CR;
            trigger StatTileByVariables;
            unset $TileByVars;
            trigger StatRoleTipFormatLabelLists;
          finish:
            %plotErrorFinish;
            put "</TreeMap>" CR;
            xdent;
        end;

      define event StatDendrogram;
          start:
            do / if ^exists(@NodeID, @ParentID, @ClusterHeight);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<Dendrogram";
            put ' nodeID="' $models[@NodeID] anchor "#" @NodeID '"' / if exists(@NodeID);
            put ' parentID="' $models[@ParentID] anchor "#" @ParentID '"' / if exists(@ParentID);
            put ' clusterHeight="' $models[@ClusterHeight] anchor "#" @ClusterHeight '"' / if exists(@ClusterHeight);
            put ' clusters="' $models[@Clusters] anchor "#" @Clusters '"' / if exists(@Clusters);
            trigger StatCommonChartAttrs;
            putq " cut=" @Cut;
            putq " treeType=" @TreeType;
            putq " dataLabelLevel=" @dataLabelLevel;
            putq " detailLevel=" @DetailLevel;
            putq " arrangement=" @Arrangement;
            putq " includeMissingColor=" @IncludeMissingColor;
            putq " cutAttrs=" @cutAttrs;
            trigger StatDataLabelVarAttrs;
            put ">" CR;
          finish:
            %plotErrorFinish;
            put "</Dendrogram>" CR;
            xdent;
        end;

      define event StatCutOpts;
        start:
          ndent;
          put "<CutOpts";
          putq " type=" @Type;
          putq " cutHeight=" @CutHeight;
          putq " nClusters=" @NClusters;
          putq " outlineAttrs=" @OutlineAttrs;
          putq " >" CR;
          xdent;
        finish:
          ndent;
          put "</CutOpts>" CR;
          xdent;
        end;

        define event StatEllipseValid;
            unset $validEllipse;
            set $validEllipse "1" /
                if any(@SemiMinor, @SemiMinorVar)
                    and any(@SemiMajor, @SemiMajorVar)
                    and any(@Slope, @SlopeVar)
                    and any(@XOrigin, @XOriginVar)
                    and any(@YOrigin, @YOriginVar);
            unset $validEllipse /
                if cmp(".", @SemiMinor) or cmp(".", @SemiMajor)
                        or cmp(".", @XOrigin) or cmp(".", @YOrigin);
        end;

        define event StatEllipse;
          start:

            trigger StatEllipseValid;
            do / if ^exists($validEllipse);
               set $ploterror "true";
               break;
            done;

            ndent;
            put "<EllipseParm";

            putq " semiMajor=" @SemiMajor;
            put ' semiMajor="' $models[@SemiMajorVar] anchor "#" @SemiMajorVar '"'
                                                    / if exists(@SemiMajorVar);

            putq " semiMinor=" @SemiMinor;
            put ' semiMinor="' $models[@SemiMinorVar] anchor "#" @SemiMinorVar '"'
                                                    / if exists(@SemiMinorVar);
            trigger StatSlopeConstVar;
            trigger StatOriginConstVars;
            trigger StatGroupVars;

            trigger StatCommonChartAttrs;
            trigger StatCurveLabelAttrs;

            put ">" CR;
          finish:
            do / if ^exists($validEllipse);
                 unset $ploterror;
                trigger StatPlotError;
                 break;
            done;
            put "</EllipseParm>" CR;
            xdent;
        end;

        define event StatMatrixEllipseList;
        start:
            ndent;
            put "<MatrixEllipseList>" CR;
        finish:
            put "</MatrixEllipseList>" CR;
            xdent;
        end;

        define event StatMatrixEllipse;
          start:
            trigger StatEllipseValid;
            break / if ^exists($validEllipse);
            ndent;
            put "<MatrixEllipseParm";
            putq " cellNumber=" @CellNumber;
            putq " semiMajor=" @SemiMajor;
            putq " semiMinor=" @SemiMinor;
            trigger StatSlopeConstVar;
            trigger StatOriginConstVars;
            /* These attrs should always be set for the matrix ellipse */
            put ' clip="true"';
            put ">" CR;
          finish:
            break / if ^exists($validEllipse);
            put "</MatrixEllipseParm>" CR;
            xdent;
        end;

        define event StatBlock;
          start:
            do / if ^exists(@XVar,@BlockVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<BlockPlot";
            put ' block="' $models[@BlockVar] "#" @BlockVar '"'  / if @BlockVar;
            putq " blockAttrVar=" @BlockAttrVar;
            put ' blockIndex="' $models[@BlockIndexVar] "#" @BlockIndexVar '"'/
                                                if exists(@BlockIndexVar);
            put ' class="' $models[@ClassVar] "#" @ClassVar '"'/
                                                if exists(@ClassVar);
            trigger StatVariableAttrs;

            trigger StatCommonChartAttrs;

            putq " altFillAttrs=" @AltFillAttrs;
            putq " fillType=" @FillType;
            putq " extendBlockOnMissing=" @ExtendBlockOnMissing;
            putq " includeMissingClass=" @IncludeMissingClass;
            putq " repeatedValues=" @RepeatedValues;
            putq " valueAttrs=" @ValueAttrs;
            putq " valueFitPolicy=" @ValueFitPolicy;
            putq " valueHAlign=" @ValueHAlign;
            putq " valueVAlign=" @ValueVAlign;
            put ">" CR;

          finish:
            do / if ^exists(@XVar, @BlockVar);
                 unset $ploterror;
                trigger StatPlotError;
                 break;
            done;
            put "</BlockPlot>" CR;
            xdent;
        end;

        define event StatBox;
          start:
            do / if ^exists(@YVar,@StatVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<BoxPlotParm";
            trigger StatVariableAttrs;

            put ' stat="' $models[@StatVar] "#" @StatVar '"';

            trigger StatDataLabelVarAttrs;

                /* Automatic dynamics - not user-set */
            putq " panelByCount=" @PanelByCount;
            putq " panelNumber=" @PanelNumber;

            trigger StatCommonChartAttrs;
            putq " boxWidth=" @BoxWidth;
            putq " capShape=" @CapShape;
            putq " connect=" @Connect;
            putq " displayStats=" @DisplayStats;
            putq " extreme=" @Extreme;
            putq " intervalBoxWidth=" @IntervalBoxWidth;
            putq " labelFar=" @LabelFar;
            putq " meanAttrs=" @MeanAttrs;
            putq " medianAttrs=" @MedianAttrs;
            putq " outlierAttrs=" @OutlierAttrs;
            putq " spread=" @Spread;
            putq " whiskerAttrs=" @WhiskerAttrs;

            putq " outlierTip=" $OutlierTipRoleStr;
            unset $OutlierTipRoleStr;

            put ">" CR;
            trigger StatRoleTipFormatLabelLists;

          finish:
            %plotErrorFinish;
            put "</BoxPlotParm>" CR;
            xdent;
        end;

        define event StatDropLine;
          start:
            do  / if ^any(@X, @XVar) or ^any(@Y, @YVar);
                set $ploterror "true";
                break;
            done;

            ndent;
            put "<DropLine";

            putq " x=" @X;
            putq " y=" @Y;

            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;

            putq " dropTo=" @DropTo;

            put ">" CR;
          finish:
            do / if $ploterror;
                trigger StatPlotError;
                unset $ploterror;
                break;
            done;
            put "</DropLine>" CR;
            xdent;
        end;

        define event StatReferenceLine;
          start:
            do / if ^any(@X, @Y, @XVar, @YVar);
               set $ploterror "true";
               break;
            done;

            ndent;
            put "<ReferenceLine";

            putq " x=" @X;
            putq " y=" @Y;

            trigger StatVariableAttrs;

            trigger StatCommonChartAttrs;
            trigger StatCurveLabelAttrs;

            put ">" CR;
          finish:
            do / if $ploterror;
                trigger StatPlotError;
                unset $ploterror;
                break;
            done;
            put "</ReferenceLine>" CR;
            xdent;
        end;

    /* NOTE - we should use ploterror instead of separate valid<plotType>.
     * It would simplify the logic. Probably not a good time right now with
     * 9.2 being 'close' 08MAR07
     */
        define event StatLineParmValid;
            unset $validLineParm;
            set $validLineParm "1" /
                if any(@X, @XVar) and any(@Y, @YVar) and any(@Slope, @SlopeVar);
            unset $validLineParm /
                if cmp(".", @X) or cmp(".", @Y);
        end;

            /* Parametric (slope+intercept) line */
        define event StatLineParm;
          start:
            trigger StatLineParmValid;
            do / if ^exists($validLineParm);
               set $ploterror "true";
               break;
            done;

            ndent;
            put "<LineParm";

            putq " y=" @Y;
            putq " x=" @X;

            trigger StatVariableAttrs;
            trigger StatSlopeConstVar;

            trigger StatCommonChartAttrs;
            trigger StatCurveLabelAttrs;

            putq " extend=" @Extend;
            put ">" CR;
          finish:
            do / if ^exists($validLineParm);
                 unset $ploterror;
                trigger StatPlotError;
                 break;
            done;
            put "</LineParm>" CR;
            xdent;
        end;

        define event StatContour;
          start:
            do / if ^exists(@XVar,@YVar,@ZVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<ContourPlotParm";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;
            putq " contourType=" @ContourType;
            putq " lineLabelAttrs=" @LineLabelAttrs;
            putq " lineLabelBaseline=" @LineLabelBaseline;
            putq " lineLabelFormat=" @LineLabelFormat;
            putq " lineLabelPosition=" @LineLabelPosition;
            putq " nHint=" @nHint;
            putq " nLevels=" @nLevels;
            putq " gridded=" @gridded;
            put ">" CR;
          finish:
            do / if ^exists(@XVar, @YVar, @ZVar);
                 unset $ploterror;
                trigger StatPlotError;
                 break;
            done;
            put "</ContourPlotParm>" CR;
            xdent;
        end;

        define event StatSurface;
          start:
            do / if ^exists(@XVar,@YVar,@ZVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<SurfacePlotParm";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;
            trigger StatCommon3DAttrs;
            putq " surfaceType=" @SurfaceType;
            put ' surfaceColorGradient="' $models[@SurfaceColorGradient]
                    anchor "#" @SurfaceColorGradient '"'
                                        / if exists(@SurfaceColorGradient);
            putq " surfaceColorGradientAttrVar=" @SurfaceColorGradientAttrVar;
            put ">" CR;
          finish:
            do / if ^exists(@XVar, @YVar, @ZVar);
                 unset $ploterror;
                trigger StatPlotError;
                 break;
            done;
            put "</SurfacePlotParm>" CR;
            xdent;
        end;

        define event StatNeedle;
          start:
            do / if ^exists(@XVar,@YVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<NeedlePlot";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;
            trigger StatDataLabelVarAttrs;
            put ">" CR;
            trigger StatRoleTipFormatLabelLists;
          finish:
            do / if ^exists(@XVar, @YVar);
                 unset $ploterror;
                trigger StatPlotError;
                 break;
            done;
            put "</NeedlePlot>" CR;
            xdent;
        end;

        define event StatPieHelper;
         start:
            break / if $ploterror;
            break / if ^exists(@ObjName);

            ndent;
            put   "<" @ObjName;

            putq " fillAttrs=" @FillAttrs;

            putq " label=" @Label;
            putq " labelAttrs=" @LabelAttrs;
            putq " location=" @Location;

            putq " percent=" @Percent;
            putq " maxSlices=" @MaxSlices;

            putq " type=" @Type;
            putq " valueAttrs=" @valueAttrs;
            put  ">" CR;

        finish:
            break / if $ploterror;
            put "</" @ObjName ">" CR;
            xdent;
        end;

        define event StatPathDiagram;
          start:
            do / if ^exists(@FromID, @ToID, @NodeX, @NodeY);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<PathDiagram";
            %putModelVar(@FromID, fromID);
            %putModelVar(@ToID, toID);
            %putModelVar(@NodeX, nodeX);
            %putModelVar(@NodeY, nodeY);

            %putModelVar(@LinkID, linkID);
            %putModelVar(@LinkLabel, linkLabel);
            %putModelVar(@LinkControlX, linkControlX);
            %putModelVar(@LinkControlY, linkControlY);
            %putModelVar(@LinkDirection, linkDirection);
            %putModelVar(@NodeID, nodeID);
            %putModelVar(@NodeTitle, nodeTitle);
            %putModelVar(@NodeDetails, nodeDetails);
            %putModelVar(@NodeWidth, nodeWidth);
            %putModelVar(@NodeHeight, nodeHeight);
            %putModelVar(@NodeShape, nodeShape);
            trigger StatCommonChartAttrs;
            putq " nodeDetailsAttrs=" @NodeDetailsAttrs;
            putq " nodeTitleAttrs=" @NodeTitleAttrs;
            put ">" CR;
          finish:
            %plotErrorFinish;
            put "</PathDiagram>" CR;
            xdent;
        end;

        define event StatPie;
          start:
            do / if ^exists(@Category);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<PieChart";
            put ' category="' $models[@Category] anchor "#" @Category '"'
                                                       / if exists(@Category);
            putq " categoryAttrVar=" @CategoryAttrVar;
            put ' response="' $models[@Response] anchor "#" @Response '"'
                                                    / if exists(@Response);
            trigger StatCommonChartAttrs;
            trigger StatGroupVars;
            trigger StatDataLabelVarAttrs;
            putq " categoryDirection=" @categoryDirection;
            putq " dataLabelContent=" @DataLabelContent;
            putq " dataLabelLocation=" @DataLabelLocation;
            putq " groupGap=" @GroupGap;
            putq " labelFitPolicy=" @LabelFitPolicy;
            putq " otherSlice=" @OtherSlice;
            putq " start=" @Start;
            putq " stat=" @Stat;
            putq " type=" @Type;
            put ">" CR;
            trigger StatRoleTipFormatLabelLists;
          finish:
            %plotErrorFinish;
            put "</PieChart>" CR;
            xdent;
        end;

        define event StatFringe;
          start:
            do / if ^exists(@XVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<FringePlot";
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;
            putq " fringeHeight=" @FringeHeight;
            put ">" CR;
            trigger StatRoleTipFormatLabelLists;
          finish:
            do / if ^exists(@XVar);
                 unset $ploterror;
                trigger StatPlotError;
                 break;
            done;
            put "</FringePlot>" CR;
            xdent;
        end;

        define event StatSeries;
          start:
            do / if ^exists(@XVar,@YVar);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<" name;
            trigger StatVariableAttrs;
            trigger StatCommonChartAttrs;
            trigger StatDataLabelVarAttrs;
            putq " break=" @Break;
            putq " connectOrder=" @ConnectOrder;
            putq " smoothConnect=" @SmoothConnect;

                /* ParetoLine opts */
            putq " anchor=" @Anchor;
                /* End ParetoLine opts */

            trigger StatCurveLabelAttrs;
            put ">" CR;
            trigger StatRoleTipFormatLabelLists;
          finish:
            %plotErrorFinish;
            put "</" name ">" CR;
            xdent;
        end;

        define event StatVectorValid;
            unset $validVector;
            set $validVector "1" /
                if exists(@XVar, @YVar)
                    and any(@XOrigin, @XOriginVar)
                    and any(@YOrigin, @YOriginVar);
            unset $validVector /
                if cmp(".", @XOrigin) or cmp(".", @YOrigin)
                    ;
        end;

        define event StatVector;
          start:
            trigger StatVectorValid;
            do / if ^exists($validVector);
               set $ploterror "true";
               break;
            done;
            ndent;
            put "<VectorPlot";
            trigger StatVariableAttrs;
            trigger StatOriginConstVars;
            trigger StatDataLabelVarAttrs;

            putq " scale=" @Scale;
            putq " shaftProtected=" @ShaftProtected;
            putq " arrowHeads=" @ArrowHeads;
            putq " arrowHeadShape=" @ArrowHeadShape;
            putq " arrowDirection=" @ArrowDirection;
            trigger StatCommonChartAttrs;
            put ">" CR;
            trigger StatRoleTipFormatLabelLists;
          finish:
            %plotErrorFinish;
            put "</VectorPlot>" CR;
            xdent;
        end;


        define event StatGraphXMLData;
            start:
               putq '<PARAM NAME="XMLDATA" VALUE=' CR;
               put "'" CR;
            finish:
               put "'" CR;
               put ">" CR;
        end;

        define event StatGraphXMLView;
            start:
               putq '<PARAM NAME="XML" VALUE=' CR;
               put "'" CR;
            finish:
               put "'" CR;
               put ">" CR;
        end;

        define event graph_border;
        end;

        define event graph_actx_width;
           style=Graph;
           pure_style;
           set $outputwidth '640';
           set $outputwidth colwidth / if ^cmp(colwidth, '0');
           set $outputwidth outputwidth / if outputwidth;
        end;

        define event graph_actx_height;
           style=Graph;
           pure_style;
           set $outputheight '480';
           set $outputheight depth / if ^cmp(depth, '0');
           set $outputheight outputheight / if outputheight;
        end;

        define event graph_fxd_attributes;
            /*pure_style;*/
            style=Graph;
            trigger graph_actx_width;
            trigger graph_actx_height;
            trigger graph_style_inline;
            put  ' ALIGN="top" class="Graph">' CR;
        end;

        define event graph_style_inline;
            put 'style="';
            put ' width: ' prxchange('s/(\.|\d)$/\1px/', -1, $outputwidth) ';' / exists($outputwidth);
            put ' height: ' prxchange('s/(\.|\d)$/\1px/', -1, $outputheight) ';' / exists($outputheight);
            put ' background-color: ' background ';' / exists(background);
            /* goptions noborder */
            put ' border-width: 0px;' / cmp(borderwidth, '0px');
            put '"';
        end;

        define event graph_java_width;
           style=Graph;
           pure_style;
           set $outputwidth '640';
           set $outputwidth colwidth / if ^cmp(colwidth, '0');
           set $outputwidth outputwidth / if outputwidth;
        end;

        define event graph_java_height;
           style=Graph;
           pure_style;
           set $outputheight '480';
           set $outputheight depth / if ^cmp(depth, '0');
           set $outputheight outputheight / if outputheight;
        end;

        define event graph_java2_attribute;
           put '   document.writeln("';
           put NAME '=\"' VALUE '\"';
           put '");' CR;
        end;

        define event graph_java2_fxd_attributes;
           put '   document.writeln(">");' CR;
           trigger javascript finish;
        end;

        define event graph_attribute;
            put " " NAME;
            putq "=" VALUE CR;
        end;

        define event graph_parameter;
            putq "<PARAM NAME=" NAME;
            putq " VALUE=" VALUE;
            put ">" CR;
        end;

        define event graph_onload;
        end;

        define event graph_fillin_block;
        end;

        define event graph_fxd_parameters;
            putq '<PARAM NAME="BackColor" VALUE=' BACKGROUND;
            put ">" CR / exists(BACKGROUND );
        end;

        define event html3_center;
            start:
               put "<CENTER>" CR;
            finish:
               put "</CENTER>" CR;
        end;

        define event java_unsupported;
            put "Sorry, your browser does not support the applet tag." CR;
            putq 'The graph ' value ' cannot be displayed.' CR;
        end;

        define event activex_unsupported;
            put "Sorry, there was a problem with the Graph control or java plug-in in your browser." CR;
            putq 'The graph ' value ' cannot be displayed.' CR;
        end;

        define event graph_description;
            putq value CR;
        end;

        define event java2_parameters;
                put  '<PARAM NAME="TYPE" VALUE="application/x-java-applet;version=1.6">' CR;
                put  '<PARAM NAME="SCRIPTABLE" VALUE="true">' CR;
        end;

        /* This event is to allow the xmldoc being driven by graph */
        /* to be able to write out an ODSTAG string to the HTML    */
        /* document. Do not remove this event unless you find out  */
        /* that EG does not need ODSTAG strings any more. --sandy  */

        define event odstag_eg;
            put "<!-- ODSTAG TYPE=" value ;
            put " ID=" anchor ;
            put  " -->" CR;
        end;


        define event java2_graph;
            start:
                pure_style;
                style=Graph;
                trigger ie_check / if ^exists($ieCheckDone);
                trigger javascript start;
                put  '   document.writeln("<OBJECT");' CR;
                put  '   document.writeln(''';
                trigger graph_java_width;
                trigger graph_java_height;
                trigger graph_style_inline;
                put  ''');' CR;
                put  '   document.writeln("ALIGN=\"baseline\" class=\"Graph\"");' CR;
                put  'if (_ie == true) {' CR;
                put  '   document.writeln("CLASSID=\"clsid:8AD9C840-044E-11D1-B3E9-00805F499D93\"");' CR;
                do / if CODEBASE;
                put  '   document.writeln("CODEBASE=\"' CODEBASE '\"");' CR;
                else;
                put  '   if (_ie64 == true) {' CR;
                put  '      document.writeln("CODEBASE=\"http://javadl.sun.com/webapps/download/AutoDL?BundleId=47155#Version=1,6,0,0\"");' CR;
                put  '   }' CR;
                put  '   else {' CR;
                put  '      document.writeln("CODEBASE=\"http://java.sun.com/update/1.6.0/jinstall-6u24-windows-i586.cab#Version=1,6,0,0\"");' CR;
                put  '   }' CR;
                done;
                put  '}' CR;
                put  'else {' CR;
                put  '   document.writeln("CODEBASE=\"http://javadl.sun.com/webapps/download/GetFile/1.6.0_24-b07/windows-i586/xpiinstall.exe\"");' CR;
                put  '   document.writeln("TYPE=\"application/x-java-applet;version=1.6\"");' CR;
                put  '}' CR;

            finish:
                put  '</OBJECT>' CR;
            end;

        define event javascript;
         start:
             put '<script language="javascript" type="text/javascript">' CR;
             put "<!-- " CR;
         finish:
             put CR "//-->"  CR;
             put "</script>" CR CR;
             put '<noscript></noscript>' CR;               /* 508 */
        end;

    end;
    %end;  %mend;
    %tagsets_statgraph;

    /*******************************************************/
    /*******************************************************/
    /*******************************************************/
    /*  GRAPH -- XML output to render graphs in HTML, etc. */
    /*******************************************************/
    /*******************************************************/
    /*******************************************************/

   %macro tagsets_graph;

    %if  %substr(&sysver,1,1) = 8 %then %do;
       /* not in version 8 */
       %end;

    %else %do;

    define tagset tagsets.graph;
        private;
        notes "This is the Graph XML definition";

        /* Macro variables for BIP testing */
        mvar StackedBarsEnabled;
        mvar UniformBarsEnabled;
        mvar ShowPieGroupHeadings;
        mvar ShowPieSubgroupHeading;
        mvar ShowPieResponseHeading;
        mvar PieGroupLabelPosition;
        mvar PieSubgroupLabelPosition;

        parent= tagsets.cascading_stylesheet;

        %let  map =<>%nrstr(&)%str(%')%str(%");
        map="&map";
        mapsub = '/&lt;/&gt;/&amp;/&#39;/&quot;/';
        nobreakspace = ' ';
        split = '<br/>';
        output_type = 'xml';
        indent=2;

      define event unicode ;
            Notes "Unicode function inserts unicode values. ";
            start:
               unset $newvalue;
               eval $temp value ;
               set  $temp strip($temp);
               set  $temp upcase($temp);

               do / if  $unicodeMap[$temp] ;
                     set $newvalue $unicodeMap[$temp] ;
                     put '&#x' $newvalue ';';
               else;
                  set $sglq "'";
                  set $dblq '"';
                  do / if (index($temp, "X") > 0);
                        do / if (index($temp, $sglq) > 0);
                           set $newvalue scan($temp, 1, "'");
                        else;
                           do / if (index($temp, $dblq) > 0);
                              set $newvalue scan($temp, 1, '"');
                           done;
                        done;
                        do / if $newvalue;
                           put '&#x' $newvalue ';';
                        done;
                  done ;
            done ;
         end;

        define event GraphXMLParam;
           start:
              put '<PARAM NAME="XMLPARM' ref_id '"';
              put " VALUE='" CR;
           finish:
              put "'" CR;
              put ">" CR;
        end;

        define event GraphData;
          start:
            ndent;
            put "<Data";
            put ">" CR;
          finish:
            put "</Data>" CR;
            xdent;
        end;

        define event IQGraphData;
          start:
            ndent;
            put '<IQData type="2DEmbedded"';
            put ' name="GraphData';
            put anchor '"';
            put ">" CR;
          finish:
            put "</IQData>" CR;
            xdent;
        end;

        define event EnterpriseData;
          start:
            ndent;
            put '<EnterpriseData';
            put ' name="GraphData';
            put anchor '" ';
            putq " server=" @GraphDataServer;
            putq " file=" @GraphDataFile;
            putq " engine=" @GraphDataEngine;
            put ">" CR;
          finish:
            put "</EnterpriseData>" CR;
            xdent;
        end;

        define event Variables;
          start:
            ndent;
            put "<Variables>" CR;
          finish:
            put "</Variables>" CR;
            xdent;
        end;

        define event Variable;
          start:
            ndent;
            put "<Variable";
            putq " SASFormat=" @Format;
            putq " encoding=" @Encoding;
            putq " name=" ref_id;
            putq " varName=" name;
            putq " label=" label;
            putq " type=" type;
            put "/>" CR;
            xdent;
        end;

        define event EmbeddedData;
          start:
            ndent;
            put "<EmbeddedData>" CR;
          finish:
            put "</EmbeddedData>" CR;
            xdent;
        end;

        define event ValuesList;
          start:
            ndent;
            put "<ValuesList>" CR;
          finish:
            put "</ValuesList>" CR;
            xdent;
        end;

        define event DataValues;
          start:
            ndent;
            put "<Values>" CR;
          finish:
            put "</Values>" CR;
            xdent;
        end;

        define event DataValue;
          start:
            ndent;
            put "<Value";
            putq " missing=" missing;
            putq " null=" @Empty;
            put ">";
            put value;
          finish:
            put "</Value>" CR;
            xdent;
        end;

        define event DataModelValues;
        end;

        define event DataModelValue;
          start:
            ndent;
            put "<V";
            putq " missing=" missing;
            putq " null=" @Empty;
            put ">";
            put value;
          finish:
            put "</V>" CR;
            xdent;
        end;


        define event ChartDataList;
          start:
            ndent;
            put "<ChartDataList>" CR;
          finish:
            put "</ChartDataList>" CR;
            xdent;
        end;



        define event ChartData;
          start:
            ndent;
            put "<ChartData";
            putq " name=" ref_id;
            putq " collation=" @Collation;
            put ' data="GraphData';
            put anchor '"';
            put ">" CR;
          finish:
            put "</ChartData>" CR;
            xdent;
        end;



        define event Roles;
          start:
            ndent;
            put "<Roles>" CR;
          finish:
            put "</Roles>" CR;
            xdent;
        end;



        define event Role;
          start:
            ndent;
            put "<Role";
            putq " variable=" @VariableID;
            putq " type=" type;
            putq " statistic=" @Statistic;
            putq " otherEnabled=" @OtherEnabled;
            putq " otherType=" @OtherType;
            putq " otherValue=" @OtherValue;
            putq " otherLabel=" @otherlabel;
            putq " sortOrder=" @SortOrder;
            putq " missingEnabled=" @MissingEnabled;
            put ">" CR;
          finish:
            put "</Role>" CR;
            xdent;
        end;



        define event Categorization;
          start:
            ndent;
            put "<Categorization";
            putq " type=" type;
            putq " numberOfLevels=" @NumberOfLevels;
            putq " rangeFormat=" @RangeFormat;
            put ">" CR;
          finish:
            put "</Categorization>" CR;
            xdent;
        end;



        define event Ranges;
          start:
            ndent;
            put "<Ranges>" CR;
          finish:
            put "</Ranges>" CR;
            xdent;
        end;



        define event range;
          start:
            ndent;
            put "<Range";
            putq " value=" value;
            putq " low=" @Low;
            putq " high=" @High;
            put ">" CR;
          finish:
            put "</Range>" CR;
            xdent;
        end;

        define event GraphSASReport;
          start:
            put '<?xml version="1.0"';
            putq ' encoding=' ENCODING / if exists(ENCODING);
            put ' ?>' CR;
            put  '<SASReport version="1.0" xmlns:style="http://www.sas.com/sasreportmodel/styles">' CR;
          finish:
            put "</SASReport>" CR;
        end;


        define event GraphView;
          start:
            ndent;
            put "<View";
            put ' style:scheme="GraphStyleScheme';
            put anchor '"';
            put ">" CR;
          finish:
            put "</View>" CR;
            xdent;
        end;

        define event GraphSection;
          start:
            ndent;
            put '<Section>' CR;
          finish:
            put "</Section>" CR;
            xdent;
        end;

        define event GraphBody;
          start:
            ndent;
            put '<Body>' CR;
          finish:
            put "</Body>" CR;
            xdent;
        end;

/***********************************************************************/
/*                   Graph appearance scheme events.                   */
/***********************************************************************/

        define event GraphLineStyles;
             break / if !exist($linestyle);
             eval $lsnum inputn($linestyle,'2.');
             set $linestyle "1" / if $lsnum < 1 or $lsnum > 46;

            /******************************************************************/
            /* It's probably more efficient to do numeric comparisons than    */
            /* Doing compares.  But it's not something I'm willing to change  */
            /* at this point.                                                 */
            /******************************************************************/

             put ' lineStyle="Solid"' / if cmp($linestyle, "1");
             put ' lineStyle="Dash1"' / if cmp($linestyle, "2");
             put ' lineStyle="Dash2"' / if cmp($linestyle, "3");
             put ' lineStyle="Dash3"' / if cmp($linestyle, "4");
             put ' lineStyle="Dash4"' / if cmp($linestyle, "5");
             put ' lineStyle="Dash5"' / if cmp($linestyle, "6");
             put ' lineStyle="Dash6"' / if cmp($linestyle, "7");
             put ' lineStyle="Dash3Dash1"' / if cmp($linestyle, "8");
             put ' lineStyle="Dash4Dash1"' / if cmp($linestyle, "9");
             put ' lineStyle="Dash5Dash1"' / if cmp($linestyle, "10");
             put ' lineStyle="Dash6Dash1"' / if cmp($linestyle, "11");
             put ' lineStyle="Dash5Dash3"' / if cmp($linestyle, "12");
             put ' lineStyle="Dash6Dash3"' / if cmp($linestyle, "13");
             put ' lineStyle="Dash3Dash1Dash1"' / if cmp($linestyle, "14");
             put ' lineStyle="Dash4Dash1Dash1"' / if cmp($linestyle, "15");
             put ' lineStyle="Dash5Dash1Dash1"' / if cmp($linestyle, "16");
             put ' lineStyle="Dash6Dash1Dash1"' / if cmp($linestyle, "17");
             put ' lineStyle="Dash5Dash3Dash3"' / if cmp($linestyle, "18");
             put ' lineStyle="Dash6Dash3Dash3"' / if cmp($linestyle, "19");
             put ' lineStyle="Dash2Space"' / if cmp($linestyle, "20");
             put ' lineStyle="Dash3Space"' / if cmp($linestyle, "21");
             put ' lineStyle="Dash4Space"' / if cmp($linestyle, "22");
             put ' lineStyle="Dash5Space"' / if cmp($linestyle, "23");
             put ' lineStyle="Dash6Space"' / if cmp($linestyle, "24");
             put ' lineStyle="Dash3Dash2"' / if cmp($linestyle, "25");
             put ' lineStyle="Dash4Dash2"' / if cmp($linestyle, "26");
             put ' lineStyle="Dash5Dash2"' / if cmp($linestyle, "27");
             put ' lineStyle="Dash6Dash2"' / if cmp($linestyle, "28");
             put ' lineStyle="Dash3Dash2Dash2"' / if cmp($linestyle, "29");
             put ' lineStyle="Dash4Dash2Dash2"' / if cmp($linestyle, "30");
             put ' lineStyle="Dash5Dash2Dash2"' / if cmp($linestyle, "31");
             put ' lineStyle="Dash6Dash2Dash2"' / if cmp($linestyle, "32");
             put ' lineStyle="Dot1"' / if cmp($linestyle, "33");
             put ' lineStyle="Dot1Space"' / if cmp($linestyle, "34");
             put ' lineStyle="Dot1Space2"' / if cmp($linestyle, "35");
             put ' lineStyle="Dot2"' / if cmp($linestyle, "36");
             put ' lineStyle="Dot3"' / if cmp($linestyle, "37");
             put ' lineStyle="Dot2Dot1"' / if cmp($linestyle, "38");
             put ' lineStyle="Dot3Dot1"' / if cmp($linestyle, "39");
             put ' lineStyle="Dot3Dot2"' / if cmp($linestyle, "40");
             put ' lineStyle="Dash3Dot1"' / if cmp($linestyle, "41");
             put ' lineStyle="Dash3Dot2"' / if cmp($linestyle, "42");
             put ' lineStyle="Dash3Dot3"' / if cmp($linestyle, "43");
             put ' lineStyle="Dash3Dot2Dash3Dot1"' / if cmp($linestyle, "44");
             put ' lineStyle="Dash3Dot3Dash3Dot1"' / if cmp($linestyle, "45");
             put ' lineStyle="Dash3Dot3Dash3Dot2"' / if cmp($linestyle, "46");
             unset $linestyle;
          end;


        define event chart_transparency;
          style=GraphCharts;
          pure_style;
          putq " chartTransparency=" TRANSPARENCY /
               if exist(TRANSPARENCY) and
                  0 <= inputn(TRANSPARENCY, '4.2') <= 1;
        end;

        define event wall_transparency;
          style=GraphWalls;
          pure_style;
          putq " wallTransparency=" TRANSPARENCY /
               if exist(TRANSPARENCY) and
                  0 <= inputn(TRANSPARENCY, '4.2') <= 1;
        end;

        define event legend_transparency;
          style=GraphLegendBackground;
          pure_style;
          putq " legendTransparency=" TRANSPARENCY /
               if exist(TRANSPARENCY) and
                  0 <= inputn(TRANSPARENCY, '4.2') <= 1;
        end;

        define event graphscheme;
          start:
            ndent;
            put '<style:Style name="GraphScheme';
            put anchor '"';
            putq " numDataAppearances=" @NumDataAppearances;
            putq " colorBy=" @ColorBy;
            trigger chart_transparency;
            trigger wall_transparency;
            trigger legend_transparency;
            put ">" CR;
          finish:
            put "</style:Style>" CR;
            xdent;
        end;

        define event LineThickness;
            putq " lineThickness=" OUTPUTWIDTH / if !exists(LINETHICKNESS);
            putq " lineThickness=" LINETHICKNESS / if !exists(OUTPUTWIDTH);
            putq " lineThickness=" LINETHICKNESS / if exists(OUTPUTWIDTH,LINETHICKNESS);
        end;

        define event AxisLineScheme;
          start:
            style=GraphAxisLines;
            pure_style;
            ndent;
            break / if !any(FOREGROUND, LINESTYLE, OUTPUTWIDTH, LINETHICKNESS );
            put '<style:Style context="AxisLines"';
            putq " lineColor=" FOREGROUND;
            set $linestyle LINESTYLE / if exists(LINESTYLE);
            trigger GraphLineStyles / if exists(LINESTYLE);
            trigger LineThickness;
            put "/>" CR;
          finish:
            xdent;
        end;

        define event GridLineScheme;
          start:
            style=GraphGridLines;
            pure_style;
            ndent;
            break / if !any(FOREGROUND, LINESTYLE, OUTPUTWIDTH, LINETHICKNESS);
            put '<style:Style context="GridLines"';
            putq " lineColor=" FOREGROUND;
            set $linestyle LINESTYLE / if exists(LINESTYLE);
            trigger GraphLineStyles / if exists(LINESTYLE);
            trigger LineThickness;
            put "/>" CR;
          finish:
            xdent;
        end;

        define event ReferenceLineScheme;
          start:
            style=GraphReference;
            pure_style;
            ndent;
            break / if !any(CONTRASTCOLOR, LINESTYLE, OUTPUTWIDTH, LINETHICKNESS);
            put '<style:Style context="ReferenceLines"';
            putq " lineColor=" CONTRASTCOLOR;
            set $linestyle LINESTYLE / if exists(LINESTYLE);
            trigger GraphLineStyles / if exists(LINESTYLE);
            trigger LineThickness;
            put "/>" CR;
          finish:
            xdent;
        end;

        define event OutlineScheme;
          start:
            style=GraphOutlines;
            pure_style;
            ndent;
            break / if !any(FOREGROUND, LINESTYLE, OUTPUTWIDTH, LINETHICKNESS);
            put '<style:Style context="Outlines"';
            putq " lineColor=" FOREGROUND;
            set $linestyle LINESTYLE / if exists(LINESTYLE);
            trigger GraphLineStyles / if exists(LINESTYLE);
            trigger LineThickness;
            put "/>" CR;
          finish:
            xdent;
        end;

        define event BorderLineScheme;
          start:
            style=GraphBorderLines;
            pure_style;
            ndent;
            break / if !any(FOREGROUND, LINESTYLE, OUTPUTWIDTH, LINETHICKNESS);
            put '<style:Style context="BorderLines"';
            putq " lineColor=" FOREGROUND;
            set $linestyle LINESTYLE / if exists(LINESTYLE);
            trigger GraphLineStyles / if exists(LINESTYLE);
            trigger LineThickness;
            put "/>" CR;
          finish:
            xdent;
        end;

        define event TitleTextScheme;
          start:
            style=GraphLabelText;
            pure_style;
            ndent;
            break / if !any(DROPSHADOW, @TextColor, FOREGROUND, @TextSize, FONT_SIZE, @TextFont, FONT_FACE, FONT_STYLE, FONT_WEIGHT, @FontWeight, @FontStyle, @TextDecoration);
            put '<style:Style context="TitleText"';
            put  ' textDropShadowEnabled = "true"' / if DROPSHADOW;
            putq " textColor=" @TextColor;
            putq " textColor=" FOREGROUND / if !exists(@TextColor);
            putq " fontSize=" @TextSize;
            putq " fontSize=" FONT_SIZE / if !exists(@TextSize);
            putq " fontFamily=" @TextFont;
            putq " fontFamily=" FONT_FACE / if !exists(@TextFont);
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontWeight=" FONT_WEIGHT / if !exists(@FontWeight);
            putq " fontStyle=" @FontStyle;
            putq " fontStyle=" FONT_STYLE / if !exists(@FontStyle);
            put "/>" CR;
          finish:
            xdent;
        end;

        define event LabelTextScheme;
          start:
            style=GraphLabelText;
            pure_style;
            ndent;
            break / if !any(DROPSHADOW, @TextColor, FOREGROUND, @TextSize, FONT_SIZE, @TextFont, FONT_FACE, FONT_STYLE, FONT_WEIGHT, @FontWeight, @FontStyle, @TextDecoration);
            put '<style:Style context="LabelText"';
            put  ' textDropShadowEnabled = "true"' / if DROPSHADOW;
            putq " textColor=" @TextColor;
            putq " textColor=" FOREGROUND / if !exists(@TextColor);
            putq " fontSize=" @TextSize;
            putq " fontSize=" FONT_SIZE / if !exists(@TextSize);
            putq " fontFamily=" @TextFont;
            putq " fontFamily=" FONT_FACE / if !exists(@TextFont);
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontWeight=" FONT_WEIGHT / if !exists(@FontWeight);
            putq " fontStyle=" @FontStyle;
            putq " fontStyle=" FONT_STYLE / if !exists(@FontStyle);
            put "/>" CR;
          finish:
            xdent;
        end;

        define event ValueTextScheme;
          start:
            style=GraphValueText;
            pure_style;
            ndent;
            break / if !any(DROPSHADOW, @TextColor, FOREGROUND, @TextSize, FONT_SIZE, @TextFont, FONT_FACE, FONT_STYLE, FONT_WEIGHT, @FontWeight, @FontStyle, @TextDecoration);
            put '<style:Style context="ValueText"';
            put  ' textDropShadowEnabled = "true"' / if DROPSHADOW;
            putq " textColor=" @TextColor;
            putq " textColor=" FOREGROUND / if !exists(@TextColor);
            putq " fontSize=" @TextSize;
            putq " fontSize=" FONT_SIZE / if !exists(@TextSize);
            putq " fontFamily=" @TextFont;
            putq " fontFamily=" FONT_FACE / if !exists(@TextFont);
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontWeight=" FONT_WEIGHT / if !exists(@FontWeight);
            putq " fontStyle=" @FontStyle;
            putq " fontStyle=" FONT_STYLE / if !exists(@FontStyle);
            put "/>" CR;
          finish:
            xdent;
        end;

        define event TitleTextScheme;
          start:
            style=GraphTitleText;
            pure_style;
            ndent;
            break / if !any(DROPSHADOW, @TextColor, FOREGROUND, @TextSize, FONT_SIZE, @TextFont, FONT_FACE, FONT_STYLE, FONT_WEIGHT, @FontWeight, @FontStyle, @TextDecoration);
            put '<style:Style context="TitleText"';
            put  ' textDropShadowEnabled = "true"' / if DROPSHADOW;
            putq " textColor=" @TextColor;
            putq " textColor=" FOREGROUND / if !exists(@TextColor);
            putq " fontSize=" @TextSize;
            putq " fontSize=" FONT_SIZE / if !exists(@TextSize);
            putq " fontFamily=" @TextFont;
            putq " fontFamily=" FONT_FACE / if !exists(@TextFont);
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontWeight=" FONT_WEIGHT / if !exists(@FontWeight);
            putq " fontStyle=" @FontStyle;
            putq " fontStyle=" FONT_STYLE / if !exists(@FontStyle);
            put "/>" CR;
          finish:
            xdent;
        end;

        define event FootnoteTextScheme;
          start:
            style=GraphFootnoteText;
            pure_style;
            ndent;
            break / if !any(DROPSHADOW, @TextColor, FOREGROUND, @TextSize, FONT_SIZE, @TextFont, FONT_FACE, FONT_STYLE, FONT_WEIGHT, @FontWeight, @FontStyle, @TextDecoration);
            put '<style:Style context="FootnoteText"';
            put  ' textDropShadowEnabled = "true"' / if DROPSHADOW;
            putq " textColor=" @TextColor;
            putq " textColor=" FOREGROUND / if !exists(@TextColor);
            putq " fontSize=" @TextSize;
            putq " fontSize=" FONT_SIZE / if !exists(@TextSize);
            putq " fontFamily=" @TextFont;
            putq " fontFamily=" FONT_FACE / if !exists(@TextFont);
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontWeight=" FONT_WEIGHT / if !exists(@FontWeight);
            putq " fontStyle=" @FontStyle;
            putq " fontStyle=" FONT_STYLE / if !exists(@FontStyle);
            put "/>" CR;
          finish:
            xdent;
        end;

        define event DataTextScheme;
          start:
            style=GraphDataText;
            pure_style;
            ndent;
            break / if !any(DROPSHADOW, @TextColor, FOREGROUND, @TextSize, FONT_SIZE, @TextFont, FONT_FACE, FONT_STYLE, FONT_WEIGHT, @FontWeight, @FontStyle, @TextDecoration);
            put '<style:Style context="DataText"';
            put  ' textDropShadowEnabled = "true"' / if DROPSHADOW;
            putq " textColor=" @TextColor;
            putq " textColor=" FOREGROUND / if !exists(@TextColor);
            putq " fontSize=" @TextSize;
            putq " fontSize=" FONT_SIZE / if !exists(@TextSize);
            putq " fontFamily=" @TextFont;
            putq " fontFamily=" FONT_FACE / if !exists(@TextFont);
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontWeight=" FONT_WEIGHT / if !exists(@FontWeight);
            putq " fontStyle=" @FontStyle;
            putq " fontStyle=" FONT_STYLE / if !exists(@FontStyle);
            put "/>" CR;
          finish:
            xdent;
        end;

        define event FillImageFitStyle;
          start:
            style=GraphBackground;
            pure_style;
            break / if !any(JUST,VJUST);
            break / if ^exists(IMAGE);
            put ' fillImageFitStyle="';
            set $output "0";
            put 'Top' / when cmp("t", VJUST);
            set $output "1" / when cmp("t", VJUST);
            put 'Bottom' / when cmp("b", VJUST);
            set $output "1" / when cmp("b", VJUST);
            put 'Right' / when cmp("r", JUST);
            set $output "1" / when cmp("r", JUST);
            put 'Left' / when cmp("l", JUST);
            set $output "1" / when cmp("l", JUST);
            put '"' / when cmp("1", $output);
            break / when cmp("1", $output);
            put 'Center' / when cmp("c", JUST);
            put '"' / when cmp("c", JUST);
            break / when cmp("c", JUST);
            put 'Center' / when cmp("m", VJUST);
            put '"' / when cmp("m", VJUST);
            break / when cmp("m", VJUST);
          finish:
            unset $output;
        end;
        define event BackImageStyle;
            break / if exists(IMAGE);
            break / if ^exists(BACKGROUNDIMAGE);
            put ' fillImageFitStyle="Fit"';
        end;
        define event BackFillScheme;
          start:
            style=GraphBackground;
            pure_style;
            ndent;
            break /  if !any(@FillFile, @FillColor, IMAGE, BACKGROUND, BACKGROUNDIMAGE, STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put '<style:Style context="BackFill"';
            do / if (^exists(@FillColor));
               putq " fillFile=" @FillFile;
               putq " fillFile=" IMAGE / if !exists(@FillFile);
               putq " fillFile=" BACKGROUNDIMAGE / if !any(@FillFile, IMAGE);
               putq " fillImageFitStyle=" @BackImageFitStyle / if exists(@FillFile);
               trigger FillImageFitStyle / if ^exists(@FillFile);
               trigger BackImageStyle / if ^exists(@FillFile);
            done;
            putq " fillColor=" @FillColor;
            putq " fillColor=" BACKGROUND / if !exists(@FillColor);
            putq " fillGradientStartColor=" STARTCOLOR;
            putq " fillGradientEndColor=" ENDCOLOR;
            putq " fillGradientType=" GRADIENT_DIRECTION;
            putq " fillPattern=" @FillPattern;
          /*
            put ' fillType="ImageColorBlend"' / if exists(IMAGE, BACKGROUND);
            put "/>" CR / if exists(IMAGE, BACKGROUND);
            break / if exists(IMAGE, BACKGROUND);

            put ' fillType="ImageColorBlend"' / if exists(BACKGROUNDIMAGE, BACKGROUND);
            put "/>" CR / if exists(BACKGROUNDIMAGE, BACKGROUND);
            break / if exists(BACKGROUNDIMAGE, BACKGROUND);
          */
            put ' fillType="Image"' / if any(IMAGE, BACKGROUNDIMAGE, @FillFile )
                                         and ^exists(@FillColor);
            put "/>" CR / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            break / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            put ' fillType="Gradient"' / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put "/>" CR / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            break / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put ' fillType="Pattern"' / if !any(@FillFile, IMAGE, BACKGROUNDIMAGE);
            put "/>" CR;
          finish:
            xdent;
        end;

        define event WallGradientTransparency;
            style=GraphWalls;
            pure_style;
            break / if !exists(TRANSPARENCY);
            break / if !any(STARTCOLOR, ENDCOLOR);
            putq ' fillGradientTransparency="Enabled"' / if exists(STARTCOLOR, ENDCOLOR);
            break / if exists(STARTCOLOR, ENDCOLOR);
            putq ' fillGradientTransparency="StartColorOnly"' / if exists(ENDCOLOR);
            break / if exists(ENDCOLOR);
            putq ' fillGradientTransparency="EndColorOnly"' / if exists(STARTCOLOR);
            break / if exists(STARTCOLOR);
        end;


        define event WallFillScheme;
          start:
            style=GraphWalls;
            pure_style;
            ndent;
            break /  if !any(IMAGE, BACKGROUNDIMAGE, BACKGROUND, STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put '<style:Style context="WallFill"';
            putq " fillFile=" IMAGE;
            putq " fillFile=" BACKGROUNDIMAGE / if !exists(IMAGE);
            putq ' fillImageFitStyle="Fit"' / if exists(BACKGROUNDIMAGE);
            putq " fillColor=" BACKGROUND;
            putq " fillGradientStartColor=" STARTCOLOR;
            putq " fillGradientEndColor=" ENDCOLOR;
            putq " fillGradientType=" GRADIENT_DIRECTION;
            trigger WallGradientTransparency / if any(TRANSPARENCY, STARTCOLOR, ENDCOLOR);
            put ' fillType="ImageColorBlend"' / if exists(BACKGROUND, IMAGE);
            put  "/>" CR / if exists(BACKGROUND, IMAGE);
            break / if exists(BACKGROUND, IMAGE);
            put ' fillType="ImageColorBlend"' / if exists(BACKGROUND, BACKGROUNDIMAGE);
            put  "/>" CR / if exists(BACKGROUND, BACKGROUNDIMAGE);
            break / if exists(BACKGROUND, BACKGROUNDIMAGE);
            put ' fillType="Image"' / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            put "/>" CR / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            break / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            put ' fillType="Gradient"' / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put "/>" CR / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            break / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put ' fillType="Pattern"' / if !any(@FillFile, IMAGE, BACKGROUNDIMAGE);
            put "/>" CR;
          finish:
            xdent;
        end;


        define event FloorFillScheme;
          start:
            style=GraphFloor;
            pure_style;
            ndent;
            break /  if !any(IMAGE, BACKGROUNDIMAGE, BACKGROUND, STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put '<style:Style context="FloorFill"';
            put ' fillType="Gradient"' / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            putq " fillFile=" IMAGE;
            putq " fillFile=" BACKGROUNDIMAGE / if !exists(IMAGE);
            putq ' fillImageFitStyle="Fit"' / if exists(BACKGROUNDIMAGE);
            putq " fillColor=" BACKGROUND;
            putq " fillGradientStartColor=" STARTCOLOR;
            putq " fillGradientEndColor=" ENDCOLOR;
            putq " fillGradientType=" GRADIENT_DIRECTION;
            put ' fillType="ImageColorBlend"' / if exists(BACKGROUND, IMAGE);
            put  "/>" CR / if exists(BACKGROUND, IMAGE);
            break / if exists(BACKGROUND, IMAGE);
            put ' fillType="ImageColorBlend"' / if exists(BACKGROUND, BACKGROUNDIMAGE);
            put  "/>" CR / if exists(BACKGROUND, BACKGROUNDIMAGE);
            break / if exists(BACKGROUND, BACKGROUNDIMAGE);
            put ' fillType="Image"' / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            put "/>" CR / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            break / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            put ' fillType="Gradient"' / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put "/>" CR / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            break / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put ' fillType="Pattern"' / if !any(@FillFile, IMAGE, BACKGROUNDIMAGE);
            put "/>" CR;
          finish:
            xdent;
        end;

        define event LegendFillScheme;
          start:
            style=GraphLegendBackground;
            pure_style;
            ndent;
            break / if !any(IMAGE, BACKGROUND, BACKGROUNDIMAGE);
            put '<style:Style context="LegendFill"';
            putq " fillFile=" IMAGE;
            putq " fillFile=" BACKGROUNDIMAGE / if !exists(IMAGE);
            putq ' fillImageFitStyle="Fit"' / if exists(BACKGROUNDIMAGE);
            putq " fillColor=" BACKGROUND;
            put ' fillType="ImageColorBlend"' / if exists(BACKGROUND, IMAGE);
            put  "/>" CR / if exists(BACKGROUND, IMAGE);
            break / if exists(BACKGROUND, IMAGE);
            put ' fillType="ImageColorBlend"' / if exists(BACKGROUND, BACKGROUNDIMAGE);
            put  "/>" CR / if exists(BACKGROUND, BACKGROUNDIMAGE);
            break / if exists(BACKGROUND, BACKGROUNDIMAGE);
            put ' fillType="Image"' / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            put "/>" CR / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            break / if any(IMAGE, BACKGROUNDIMAGE, @FillFile );
            put ' fillType="Gradient"' / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put "/>" CR / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            break / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put ' fillType="Pattern"' / if !any(@FillFile, IMAGE, BACKGROUNDIMAGE);
            put "/>" CR;
          finish:
            xdent;
        end;


        define event DropShadowScheme;
          start:
            style=DropShadowStyle;
            pure_style;
            ndent;
            break /  if !any (FOREGROUND, TRANSPARENCY);
            put '<style:Style context="DropShadow"';
            putq " shadowColor=" FOREGROUND;
            putq " transparency=" TRANSPARENCY /
               if exist(TRANSPARENCY) and 0 <= inputn(TRANSPARENCY, '4.2') <= 1;
            put  ' xOffset="5%"'/if any (FOREGROUND, TRANSPARENCY);
            put  ' yOffset="5%"'/if any (FOREGROUND, TRANSPARENCY);
            put "/>" CR;
          finish:
            xdent;
        end;

/***********************************************************************/
/*          The end of the graph appearance scheme events.             */
/***********************************************************************/



        define event graph;
          start:
            ndent;
            put "<Graph";
            putq " name=" @graphid;
            putq " style:height=" @Height / if !cmp(@Height,"0px");
            trigger graph_styleheight / if cmp(@Height,"0px");
            putq " style:width=" @Width / if !cmp(@Width,"0px");
            trigger graph_stylewidth / if cmp(@Width,"0px");
            put ' style:style="GraphScheme';
            put anchor '" ';
            putq " showChartTips=" @ChartTips;
            putq " view2D=" @View2D;
            putq " g3dTilt=" @G3DTilt;
            putq " g3dRotate=" @G3DRotate;
            putq " chartBorder=" @ChartBorder;
            putq " drillDownEnabled=" @DrillDownEnabled;
            putq " drillDownMode=" @DrillDownMode;
            putq " rotationX=" @RotateX;
            putq " rotationY=" @RotateY;
            putq " rotationZ=" @RotateZ;
            put ' usePlotContrast="true"';
            put ">" CR;
          finish:
            put "</Graph>" CR;
            xdent;
        end;

        define event graph_stylewidth;
           style=Graph;
           pure_style;
           putq " style:width=" OUTPUTWIDTH / if exists(OUTPUTWIDTH);
           put ' style:width="640px"' / if !exists(OUTPUTWIDTH);
        end;

        define event graph_styleheight;
           style=Graph;
           pure_style;
           putq " style:height=" OUTPUTHEIGHT / if exists(OUTPUTHEIGHT);
           put ' style:height="480px"' / if !exists(OUTPUTHEIGHT);
        end;

        define event ChartAreas;
          start:
            ndent;
            put "<ChartAreas>" CR;
          finish:
            put "</ChartAreas>" CR;
            xdent;
        end;


        define event ChartArea;
          start:
            ndent;
            put "<ChartArea";
            putq " orientation=" @Orientation;
            put ">" CR;
          finish:
            put "</ChartArea>" CR;
            xdent;
        end;



        define event Walls;
          start:
            ndent;
            put "<Walls>" CR;
          finish:
            put "</Walls>" CR;
            xdent;
        end;



        define event Wall;
          start:
            style=GraphWalls;
            pure_style;
            ndent;
            put "<Wall";
            putq " name=" ref_id;
            putq " wallVisible=" @WallVisible;
            putq ' isTransparent="true"' / if exists(TRANSPARENCY);
            put ">" CR;
          finish:
            put "</Wall>" CR;
            xdent;
        end;



        define event Background;
          start:
            ndent;
            put "<Background";
            putq " fillColor=" @FillColor;
            putq " fillVisible=" @FillVisible;
            putq " fillFile=" @FillFile;
            putq " fillType=" @FillType;
            putq " fillImageFitStyle=" @FitStyle  / if exists(@FillFile);
            put "/>" NL;
            xdent;
        end;


        define event Charts;
          start:
            ndent;
            put "<Charts";
            putq " reverseOrder=" @ReverseOrder;
            put ">" CR;
          finish:
            put "</Charts>" CR;
            xdent;
        end;



        define event GraphAxes;
          start:
            ndent;
            put "<Axes>" CR;
          finish:
            put "</Axes>" CR;
            xdent;
        end;



        define event GraphAxis;
          start:
            ndent;
            put "<Axis";
            putq " name=" ref_id;
            putq " role=" @Role;
            putq " isVisible=" @Visible;
            putq " position=" @AxisPosition;
            putq " axisLength=" @AxisLength;
            putq " scale=" @AxisScale;
            putq " logarithmicBase=" @LogarithmicBase;
            putq " logarithmicPowerVisible=" @LogarithmicPowerVisible;
            putq " ticksBeginAtZero=" @TicksBeginAtZero;
            putq " minimum=" @Minimum;
            putq " maximum=" @Maximum;
            putq " majorTickCount=" @MajorTickCount;
            putq " minorTickCount=" @MinorTickCount;
            putq " startOffset=" @StartOffset;
            putq " endOffset=" @EndOffset;
            putq " reverseAxis=" @ReverseAxis;
            putq " gridPosition=" @GridPosition;
            putq " baseLineValue=" @BaseLineValue;
            putq " showGroupValueBrackets=" @BracketGroupValues;
            putq " groupSpacing=" @GroupSpacing;
            put  ' appendLabelStatistic="true"' / if cmp(@Role,"Response");
            put ">" CR;
          finish:
            put "</Axis>" CR;
            xdent;
        end;

        define event GraphTickList;
          start:
            ndent;
            put "<TickList>" CR;
          finish:
            put "</TickList>" CR;
            xdent;
        end;

        define event GraphTickValue;
            ndent;
            put "<Value>";
            put @Value;
            put "</Value>" CR;
            xdent;
        end;

        define event Grid;
          start:
            ndent;
            put "<Grid";
            putq " lineVisible=" @LineVisible;
            putq " lineStyle=" @LineStyle;
            putq " lineColor=" @LineColor;
            put "/>" NL;
            xdent;
        end;



        define event AxisReferenceLines;
          start:
            ndent;
            put "<AxisReferenceLines>" CR;
          finish:
            put "</AxisReferenceLines>" CR;
            xdent;
        end;



        define event AxisReference;
          start:
            ndent;
            put "<AxisReference";
            putq " dataValue=" @DataValue;
            putq " position=" @Position;
            put ">" CR;
          finish:
            put "</AxisReference>" CR;
            xdent;
        end;



        define event MajorTick;
          start:
            ndent;
            put "<MajorTick";
            putq " lineVisible=" @LineVisible;
            putq " lineColor=" @LineColor;
            putq " lineThickness=" @LineThickness;
            put "/>" NL;
            xdent;
        end;



        define event MinorTick;
          start:
            ndent;
            put "<MinorTick";
            putq " lineVisible=" @LineVisible;
            putq " lineColor=" @LineColor;
            putq " lineThickness=" @LineThickness;
            put "/>" NL;
            xdent;
        end;


        define event GraphLabel;
          start:
            ndent;
            put "<Label";
            putq " index=" @Index;
            putq " textVisible=" @TextVisible;
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textBaselineAngle=" @TextBaselineAngle;
            putq " textCharacterAngle=" @TextCharacterAngle;
            putq " textAlign=" @TextHorizontalAlignment;
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            put ">" @TextValue "</Label>" CR / if exists(@TextValue);
            break / if exists(@TextValue);
            put "/>" CR;
          finish:
            xdent;
        end;



        define event GraphLabels;
          start:
            ndent;
            put "<Labels";
            putq " textVisible=" @TextVisible;
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textBaselineAngle=" @TextBaselineAngle;
            putq " textCharacterAngle=" @TextCharacterAngle;
            putq " textAlign=" @TextHorizontalAlignment;
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            put "/>" CR;
            xdent;
        end;



        define event GraphGroupLabel;
          start:
            ndent;
            put "<GroupLabel";
            putq " textColor=" @TextColor;
            putq " textVisible=" @TextVisible;
            putq " fontSize=" @TextSize;
            putq " textBaselineAngle=" @TextBaselineAngle;
            putq " textCharacterAngle=" @TextCharacterAngle;
            putq " textAlign=" @TextHorizontalAlignment;
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            put ">" @TextValue "</GroupLabel>" CR / if exists(@TextValue);
            break / if exists(@TextValue);
            put "/>" CR;
            xdent;
        end;



        define event AxisValues;
          start:
            ndent;
            put "<AxisValues";
            putq " textVisible=" @TextVisible;
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textBaselineAngle=" @TextBaselineAngle;
            putq " textCharacterAngle=" @TextCharacterAngle;
            putq " textAlign=" @TextHorizontalAlignment;
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            put "/>" CR;
            xdent;
        end;



        define event GroupAxisValues;
          start:
            ndent;
            put "<GroupAxisValues";
            putq " textVisible=" @TextVisible;
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textBaselineAngle=" @TextBaselineAngle;
            putq " textCharacterAngle=" @TextCharacterAngle;
            putq " textAlign=" @TextHorizontalAlignment;
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            put "/>" CR;
            xdent;
        end;



        define event GroupValue;
          start:
            ndent;
            put "<GroupValue";
            putq " index=" @Index;
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textBaselineAngle=" @TextBaselineAngle;
            putq " textCharacterAngle=" @TextCharacterAngle;
            putq " textAlign=" @TextHorizontalAlignment;
            put ">" @TextValue "</GroupValue>" CR / if exists(@TextValue);
            break / if exists(@TextValue);
            put "/>" CR;
            xdent;
        end;



        define event Value;
          start:
            ndent;
            put "<Value";
            putq " index=" @Index;
            putq " value=" value;
            putq " missing=" missing;
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textBaselineAngle=" @TextBaselineAngle;
            putq " textCharacterAngle=" @TextCharacterAngle;
            putq " textAlign=" @TextHorizontalAlignment;
            put ">" @TextValue "</Value>" CR / if exists(@TextValue);
            break / if exists(@TextValue);
            put "/>" CR;
          finish:
            xdent;
        end;

        define event EntryLabels;
          start:
            ndent;
            put "<EntryLabels>" CR;
          finish:
            put "</EntryLabels>" CR;
            xdent;
        end;

        define event ChartAssociations;
          start:
            ndent;
            put "<ChartAssociations>" CR;
          finish:
            put "</ChartAssociations>" CR;
            xdent;
        end;

        define event Entrys;
          start:
            ndent;
            put "<Entrys>" CR;
          finish:
            put "</Entrys>" CR;
            xdent;
        end;

        define event LegendEntry;
          start:
            ndent;
            put "<Entry";
            putq " index=" @entryindex;
            put ">" CR;
          finish:
            put "</Entry>" CR;
            xdent;
        end;

        define event GraphText;
            ndent;
            put "<Text";
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textVisible=" @TextVisible;
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            put ">" @TextValue "</Text>" CR / if exists(@TextValue);
            break / if exists(@TextValue);
            put "/>" CR;
            xdent;
        end;


        define event Legends;
          start:
            ndent;
            put "<Legends>" CR;
          finish:
            put "</Legends>" CR;
            xdent;
        end;


        define event Legend;
          start:
            style=GraphLegendBackground;
            pure_style;
            ndent;
            put "<Legend";
            putq " chart=" @ChartID;
            putq " isVisible=" @Visible;
            putq " position=" @Position;
            putq " titlePosition=" @TitlePosition;
            putq " layoutOrder=" @LayoutOrder;
            putq " isOutside=" @Outside;
            putq " border=" @Border;
            putq " across=" @Across;
            putq " down=" @Down;
            putq " borderColor=" @BorderColor;
            putq " borderShadowColor=" @BorderShadowColor;
            putq ' isTransparent="true"' / if exists(TRANSPARENCY);
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textVisible=" @TextVisible;
            putq " textBaselineAngle=" @TextBaselineAngle;
            putq " textCharacterAngle=" @TextCharacterAngle;
            putq " textAlign=" @TextHorizontalAlignment;
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            put ">" CR;
          finish:
            put "</Legend>" CR;
            xdent;
        end;


        define event LegendTitle;
          start:
            break / if !any(@TextFont, @TextFlags, @TextColor, @TextVisible, @TextValue,
                            @TextBaselineAngle, @TextCharacterAngle, @TextHorizontalAlignment, @TextSize,
                            @FontWeight, @FontStyle, @TextDecoration);
            ndent;
            put "<Title";
            putq " fontFamily=" @TextFont;
            putq " textColor=" @TextColor;
            putq " textVisible=" @TextVisible;
            putq " textBaselineAngle=" @TextBaselineAngle;
            putq " textCharacterAngle=" @TextCharacterAngle;
            putq " textAlign=" @TextHorizontalAlignment;
            putq " fontSize=" @TextSize;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            put ">" @TextValue "</Title>" CR / if exists(@TextValue);
            break / if exists(@TextValue);
            put "/>" CR;
            xdent;
        end;


     /* begin annotation */
     define event Annotations;
      start:
        ndent;
        put "<Annotations>" CR;
      finish:
        put "</Annotations>" CR;
        xdent;
    end;

     define event AnnotationAssociations;
      start:
        ndent;
        put "<AnnotationAssociations>" CR;
      finish:
        put "</AnnotationAssociations>" CR;
        xdent;
    end;

    define event AnnotationAssociation;
      start:
        ndent;
        put "<AnnotationAssociation";
        putq " name=" @AnnoID;
        putq " chart=" @ChartID;
        put "/>" CR;
        xdent;
    end;

    define event AnnotationData;
      start:
        ndent;
        put "<AnnotationData";
        putq " name=" ref_id;
        put ">" CR;
      finish:
        put "</AnnotationData>" CR;
        xdent;
    end;

     define event AnnotationElement;
      start:
        ndent;
        put  "<" @AnnotationElement;
        putq " chartArea=" @ChartArea;
        putq " x=" @X;
        putq " y=" @Y;
        putq " z=" @Z;
        putq " xC=" @Xc;
        putq " yC=" @Yc;
        putq " group=" @Group;
        putq " category=" @Category;
        putq " subgroup=" @Subgroup;
        putq " xSys=" @Xsys;
        putq " ySys=" @Ysys;
        putq " zSys=" @Zsys;
        putq " hSys=" @Hsys;

        putq " baselineAngle=" @BaselineAngle;
        putq " startAngle=" @StartAngle;
        putq " rotate=" @Rotate;
        putq " radius=" @Radius;
        putq " radiusMultiplier=" @RadiusMultiplier;
        putq " markerShape=" @MarkerShape;
        putq " markerSize=" @MarkerSize;
        putq " position=" @Position;
        putq " fontFamily=" @TextFont;
        putq " textDecoration=" @TextDecoration;
        putq " fontWeight=" @FontWeight;
        putq " fontStyle=" @FontStyle;
        putq " fontSize=" @TextSize;
        putq " pattern=" @Pattern;
        putq " style=" @Style;
        putq " barLineStyle=" @BarLineStyle;
        putq " pieLineStyle=" @PieLineStyle;
        putq " lineStyle=" @LineStyle;
        putq " arrowheadSize=" @ArrowheadSize;
        putq " lineSize=" @LineSize;
        putq " lineColor=" @LineColor;
        putq " color=" @Color;
        putq " cBorder=" @Cborder;
        putq " cBox=" @Cbox;
        putq " imagePath=" @ImagePath;
        putq " html=" @Html;
        putq " when=" @When;
        putq " width=" @Width;
        put "/>" CR / if !exists(@Text);
        xdent / if !exists(@Text);
        break / if !exists(@Text);
        put ">";
        put @Text;
        put  "</" @AnnotationElement;
        put ">" CR;
        xdent;
    end;


    define event Polygons;
      start:
        ndent;
        put "<Polygons";
        putq " fillColor=" @FillColor;
        putq " pattern=" @Pattern;
        putq " lineStyle=" @LineStyle;
        putq " lineSize=" @LineSize;
        putq " lineColor=" @LineColor;
        putq " html=" @Html;
        putq " when=" @When;
        put ">" CR;
      finish:
        put "</Polygons>" CR;
        xdent;
    end;


    /* end annotation */


        define event DataStyle;
          start:
            ndent;
            put "<style:Style";
            putq " context=" ref_id;
          finish:
            put "/>" CR;
            xdent;
        end;


        define event DataStyleFill;
          start:
            break /  if !exists(FOREGROUND, IMAGE);
            ndent;
            put '<style:Style context="Fill"';
            putq " fillFile=" IMAGE;
            put  ' fillType="ImageColorBlend"';

          finish:
            break /  if !exists(FOREGROUND, IMAGE);
            put "/>" CR;
            xdent;
        end;

        define event DataFill;
          start:
            ndent;
            put "<Fill";
            trigger DataFillAttributes;
            put "/>" CR;

          finish:
            xdent;
        end;

        define event DataStyleColorAttributes;
          start:
            putq " color=" @Color;
            putq " color=" FOREGROUND / if !exist(@Color);
            putq " altColor=" @AltColor;
            putq " altColor=" CONTRASTCOLOR / if !exist(@AltColor);
        end;

        define event DataColorRamp;
          start:
            break / if !exist(STARTCOLOR, ENDCOLOR);
            ndent;
            put "<style:Style";
            putq " context=" ref_id;
            putq " fillGradientStartColor=" STARTCOLOR;
            putq " fillGradientEndColor=" ENDCOLOR;
            put "/>" CR;
            xdent;
        end;

        define event DataThreeColorRamp;
          start:
            break / if !exist(STARTCOLOR, NEUTRALCOLOR, ENDCOLOR);
            ndent;
            put "<style:Style";
            putq " context=" ref_id;
            putq " fillGradientStartColor=" STARTCOLOR;
            putq " fillGradientNeutralColor=" NEUTRALCOLOR;
            putq " fillGradientEndColor=" ENDCOLOR;
            put "/>" CR;
            xdent;
        end;

        define event DataFillAttributes;
          start:
            putq " fillFile=" @FillFile;
            putq " fillColor=" @FillColor;
            putq " fillPattern=" @FillPattern;
            putq " fillVisible=" @FillVisible;
            put ' fillType="Image"' / if exists(@FillFile);
            putq " fillImageFitStyle=" @FitStyle / if exists(@FillFile);
            break / if exists(@FillFile);
            put ' fillType="Pattern"' / if !exists(@FillFile);
        end;

        define event DataStyleFillAttributes;
          start:
            putq " fillFile=" IMAGE / if !exists(BACKGROUNDIMAGE);
            putq " fillFile=" BACKGROUNDIMAGE / if !exists(IMAGE);
            putq ' fillImageFitStyle="Fit"' / if exists(BACKGROUNDIMAGE);
            put ' fillType="ImageColorBlend"' / if exists(FOREGROUND, IMAGE);
            break / if exists(FOREGROUND, IMAGE);
            put ' fillType="ImageColorBlend"' / if exists(FOREGROUND, BACKGROUNDIMAGE);
            break / if exists(FOREGROUND, BACKGROUNDIMAGE);
            put ' fillType="Image"' / if any(IMAGE, BACKGROUNDIMAGE);
            break / if any(IMAGE, BACKGROUNDIMAGE);
            put ' fillType="Gradient"' / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            putq " fillGradientStartColor=" STARTCOLOR;
            putq " fillGradientEndColor=" ENDCOLOR;
            putq " fillGradientType=" GRADIENT_DIRECTION;
            break / if any(STARTCOLOR, ENDCOLOR, GRADIENT_DIRECTION);
            put ' fillType="Pattern"' / if !any(IMAGE, BACKGROUNDIMAGE);
        end;

        define event DataLabelAttributes;
          start:
            putq " textColor=" @TextColor;
        end;

        define event DataMarker;
          start:
            ndent;
            put "<Marker";
            trigger DataMarkerAttributes;
            put "/>" NL;
            xdent;
        end;

        define event DataMarkerAttributes;
          start:
            putq " markerColor=" @MarkerColor;
            putq " markerSymbol=" @MarkerSymbol;
            putq " markerSize=" @MarkerSize;
            putq " markerVisible=" @MarkerVisible;
        end;

        define event DataStyleMarkerAttributes;
          start:
            putq " markerSymbol=" MARKERSYMBOL;
            putq " markerSize=" MARKERSIZE;
        end;

        define event DataLines;
          start:
            ndent;
            put "<Line";
            trigger DataLineAttributes;
            put "/>" NL;
            xdent;
        end;

        define event DataLineAttributes;
          start:
            putq " lineColor=" @LineColor;
            putq " lineThickness=" @LineThickness;
            putq " lineStyle=" @LineStyle;
            putq " lineVisible=" @LineVisible;
        end;

        define event DataStyleLineAttributes;
          start:
            trigger LineThickness;
            set $linestyle LINESTYLE / if exists(LINESTYLE);
            trigger GraphLineStyles / if exists(LINESTYLE);
        end;

        define event LineAppearance;
          start:
            ndent;
            put "<Line";
            putq " lineStyle=" @LineStyle;
            putq " lineColor=" @LineColor;
            putq " lineThickness=" @LineThickness;
            putq " lineVisible=" @LineVisible;
            put "/>" NL;
            xdent;
        end;

        define event Outline;
          start:
            ndent;
            put "<Outline";
            putq " lineVisible=" @LineVisible;
            putq " lineColor=" @LineColor;
            putq " lineThickness=" @LineThickness;
            put "/>" NL;
            xdent;
        end;


        define event JoinLine;
          start:
            ndent;
            put "<JoinLine";
            putq " lineVisible=" @LineVisible;
            putq " lineColor=" @LineColor;
            putq " lineStyle=" @LineStyle;
            putq " lineThickness=" @LineThickness;
            put "/>" CR;
            xdent;
        end;


        define event EmptyOutline;
          start:
            ndent;
            put "<EmptyOutline";
            putq " lineColor=" @LineColor;
            put "/>" NL;
            xdent;
        end;


        define event ChartBorderLine;
          start:
            ndent;
            put "<ChartBorderLine";
            putq " lineColor=" @LineColor;
            putq " lineStyle=" @LineStyle;
            putq " lineThickness=" @LineThickness;
            putq " lineVisible=" @LineVisible;
            put "/>" NL;
            xdent;
        end;


        define event StyleScheme;
          start:
            ndent;
            put "<style:Scheme";
            put ' name="GraphStyleScheme';
            put anchor '"';
            put ' version=' &_g_version_;
            put ">" CR;
          finish:
            put "</style:Scheme>" CR;
            xdent;
        end;

        define event Text;
          start:
            ndent;
            put "<Text";
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            put ">" CR;
          finish:
            put "</Text>" CR;
            xdent;
        end;

        define event Interpolation;
          start:
            ndent;
            put "<Interpolation";
            putq " value=" value;
            putq " type=" type;
            putq " numericValue=" @numValue;
            putq " showNeedles=" @Needle;
            putq " regressionType=" @RegressionType;
            putq " forceIntercept=" @Intercept;
            putq " confidenceType=" @ConfidenceType;
            putq " confidenceLevel=" @ConfidenceLevel;
            putq " degree=" @Degree;
            putq " useParametric=" @Parametric;
            putq " sortData=" @Sort;
            putq " isFilled=" @Fill;
            putq " showJoined=" @Join;
            putq " stepPlacement=" @StepPlacement;
            putq " percentage=" @Percentage;
            putq " showEquation=" @Equation;
            put "/>" NL;
            xdent;
        end;



        define event PIE;
          start:
            style=GraphCharts;
            pure_style;
            ndent;
            put "<Pie";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " styleBy=" @StyleBy;
            do / if PieGroupLabelPosition;
               put ' groupLabelPosition="' PieGroupLabelPosition '"';
            else;
               putq " groupLabelPosition=" @groupLabelPosition;
            done;
            do / if PieSubgroupLabelPosition;
               put ' subgroupLabelPosition="' PieSubgroupLabelPosition '"';
            done;
            do / if ShowPieGroupHeadings;
               put ' showGroupHeadings="' ShowPieGroupHeadings '"';
            else;
               putq " showGroupHeadings=" @GroupHeading;
            done;
            do / if ShowPieSubgroupHeading;
               put ' showSubgroupHeading="' ShowPieSubgroupHeading '"';
            done;
            do / if ShowPieResponseHeading;
               put ' showResponseHeading="' ShowPieResponseHeading '"';
            done;
            putq " useJapaneseStyle=" @JapaneseStyle;
            put  ' isTransparent="true"' / if exists(TRANSPARENCY);
            putq " across=" @Across;
            putq " down=" @Down;
            putq " direction=" @Direction;
            putq " labelPosition=" @LabelPosition;
            putq " sliceLabelPosition=" @SliceLabelPosition;
            putq " percentLabelPosition=" @PercentLabelPosition;
            putq " startAngle=" @StartAngle;
            putq " matchColor=" @MatchColor;
            putq " otherColor=" @OtherColor;
            putq " holeSize=" @HoleSize;
            putq " pieSize=" @PieSize;
            put ">" CR;
          finish:
            put "</Pie>" CR;
            xdent;
        end;


        define event Detail;
          start:
            ndent;
            put "<Detail";
            putq " sliceLabel=" @DetailSlice;
            putq " valueLabel=" @DetailValue;
            putq " percentLabel=" @DetailPercent;
            putq " threshold=" @DetailThreshold;
            putq " radius=" @DetailRadius;
          finish:
            put "/>" CR;
            xdent;
        end;


        define event BAR;
          start:
            style=GraphCharts;
            pure_style;
            ndent;
            put "<Bar";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " styleBy=" @StyleBy;
            putq " showBaseline=" @ShowBaseline;
            putq " barShape=" @BarShape;
            put  ' isTransparent="true"' / if exists(TRANSPARENCY);
            putq " includeZeroHeightBars=" @IncludeZeroHeightBars;
            putq " calculatePercentageByGroup=" @CalculatePercentageByGroup;
            putq " barWidth=" @BarWidth;
            putq " barSpace=" @BarSpace;
            putq " errorBarShape=" @ErrorBarShape;
            putq " errorBarConfidenceLimits=" @ErrorBarConfidenceLimits;
            putq " labelStatistic=" @LabelStatistic;
            putq " labelPosition=" @LabelPosition;
            do / if StackedBarsEnabled;
               put ' stack="' StackedBarsEnabled '"';
            done;
            do / if UniformBarsEnabled;
               put ' uniform="' UniformBarsEnabled '"';
            done;
            put ">" CR;
          finish:
            put "</Bar>" CR;
            xdent;
        end;

        define event AreaBar;
          start:
            ndent;
            put "<AreaBar";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            put ">" CR;
          finish:
            put "</AreaBar>" CR;
            xdent;
        end;

        define event KPI;
          start:
            ndent;
            put "<KPI";
            putq " name=" ref_id;
            putq " type=" @KPIType;
            putq " actualLabel=" @ActualLabel;
            putq " targetValue=" @TargetValue;
            putq " actualValue=" @ActualValue;
            putq " orientation=" @orientation;
            putq " sasFormat=" @format;
            putq " boundaryInHigherSegment=" @BoundaryInHigherSegment;
            putq " renderType=" @RenderType ;
             put ">" CR;
          finish:
            put "</KPI>" CR;
            xdent;
        end;


        define event ActualLabelText;
          start:
            ndent;
            put "<ActualLabelText";
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textAlign=" @TextAlign;
            putq " textVisible=" @TextVisible ">";
            put  "</ActualLabelText>" CR;
           xdent;
        end;

        define event ValueText;
          start:
            ndent;
            put "<ValueText";
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textVisible=" @TextVisible ">";
            put  "</ValueText>" CR;
           xdent;
        end;

        define event RangeText;
          start:
            ndent;
            put "<RangeText";
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            putq " fontSize=" @TextSize;
            putq " textColor=" @TextColor;
            putq " textVisible=" @TextVisible ">";
            put  "</RangeText>" CR;
            xdent;
        end;

        define event RangeValues;
          start:
            ndent;
            put "<RangeValues>" CR;
          finish:
            put "</RangeValues>" CR;
            xdent;
        end;

        define event RangeValue;
          start:
            ndent;
            put "<RangeValue>";
            put value;
          finish:
            put "</RangeValue>" CR;
            xdent;
        end;


        define event ActiveColors;
          start:
            ndent;
            put "<ActiveColors>" CR;
          finish:
            put "</ActiveColors>" CR;
            xdent;
        end;

         define event ActiveColor;
          start:
            ndent;
            put "<ActiveColor>";
            put value;
          finish:
            put "</ActiveColor>" CR;
            xdent;
        end;

        define event RangeColors;
          start:
            ndent;
            put "<RangeColors>" CR;
          finish:
            put "</RangeColors>" CR;
            xdent;
        end;

        define event RangeColor;
          start:
            ndent;
            put "<RangeColor>";
            put value;
          finish:
            put "</RangeColor>" CR;
            xdent;
        end;

       define event Tile;
          start:
            ndent;
            put "<Tile";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " detailLevel=" @DetailLevel;
            putq " layout=" @TileLayout;
            putq " labelLevel=" @LabelLevel;
            putq " baseLine=" @Baseline;
            putq " missingColor=" @MissingColor;
            putq " isSymmetric=" @IsSymmetric;
            putq " minLegendValue=" @Minlegendvalue;
            putq " maxLegendValue=" @Maxlegendvalue;
            putq " colorType=" @Colortype;
            put ">" CR;
          finish:
            put "</Tile>" CR;
            xdent;
        end;

        define event ColorRamps;
          start:
            ndent;
            put "<ColorRamps>" CR;
          finish:
            put "</ColorRamps>" CR;
            xdent;
        end;

        define event ColorRamp;
          start:
            ndent;
            put "<ColorRamp";
            putq " color=" @RampColor;
            putq " normalizedValue=" @NormalizedValue;
            put ">";
          finish:
            put "</ColorRamp>" CR;
            xdent;
        end;

        define event Radar;
          start:
            ndent;
            put "<Radar";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " starType=" @StarType;
            putq " variableType=" @SpokeScale;
            putq " startAngle=" @StartAngle;
            putq " circularGraphType=" @CircularGraphType;
            putq " sectorType=" @SectorType;
            putq " sectorWidth=" @SectorWidth;
            putq " cellSpacing=" @CellSpacing;
            putq " numberOfRows=" @NumberOfRows;
            putq " numberOfColumns=" @NumberOfColumns;
            putq " textMode=" @TextMode;
            putq " makeSpider=" @Spider;
            putq " makeNoZero=" @NoZero;
            putq " starLegend=" @StarLegend;
            put ">" CR;
          finish:
            put "</Radar>" CR;
            xdent;
        end;


        define event TopFill;
          start:
            ndent;
            put "<TopFill" ;
            putq " fillColor=" @FillColor;
            put "/>" CR;
          finish:
            xdent;
        end;

        define event SideFill;
          start:
            ndent;
            put "<SideFill" ;
            putq " fillColor=" @FillColor;
            put "/>" CR;
          finish:
            xdent;
        end;

        define event TileFills;
          start:
            ndent;
            put  "<TileFills>" CR;
          finish:
            put  "</TileFills>" CR;
            xdent;
        end;

        define event TileFill;
          start:
            ndent;
            put "<TileFill";
            putq " tileIndex=" @TileIndex;
            put ">" CR;
          finish:
            put "</TileFill>" CR;
            xdent;
        end;

        define event ReferenceCircles;
          start:
            ndent;
            put "<ReferenceCircles>" CR;
          finish:
            put "</ReferenceCircles>" CR;
            xdent;
        end;

        define event ReferenceCircle;
          start:
            ndent;
            put "<ReferenceCircle";
            putq " circleRadius=" @CircleRadius;
            put ">" CR;
          finish:
            put "</ReferenceCircle>" CR;
            xdent;
        end;

        define event CircleLine;
          start:
            break / if !any(@LineStyle, @LineThickness, @LineColor, @LineVisible);
            ndent;
            put "<CircleLine";
            putq " lineStyle=" @LineStyle;
            putq " lineThickness=" @LineThickness;
            putq " lineColor=" @LineColor;
            putq " lineVisible=" @LineVisible;
            put "/>" NL;
            xdent;
        end;

        define event Histogram;
          start:
            ndent;
            put "<Histogram";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            put ">" CR;
          finish:
            put "</Histogram>" CR;
            xdent;
        end;


        define event HighLoPlot;
          start:
            ndent;
            put "<HighLow";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " styleBy=" @StyleBy;
            putq " responseAxis=" @ResponseAxis;
            putq " boxChartType=" @BoxChartType;
            putq " highLowType=" @HighLowType;
            putq " boxStyle=" @BoxStyle;
            putq " boxCapStyle=" @BoxCapStyle;
            putq " boxWidth=" @BoxWidth;
            putq " excludeData=" @ExcludeData;
            putq " numberOfStandardDeviations=" @NumberOfStandardDeviations;
            putq " boundaryPercentile=" @BoundaryPercentile;
            putq " computeStandardErrorOfMean=" @StandardErrorOfMean;
            putq " usePooledVariances=" @PooledVariances;
            putq " labelPosition=" @LabelPosition;
            put ">" CR;
          finish:
            put "</HighLow>" CR;
            xdent;
        end;

        define event BoxPlot;
          start:
            ndent;
            put "<Box";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " styleBy=" @StyleBy;
            putq " responseAxis=" @ResponseAxis;
            putq " boxChartType=" @BoxChartType;
            putq " highLowType=" @HighLowType;
            putq " boxStyle=" @BoxStyle;
            putq " boxCapStyle=" @BoxCapStyle;
            putq " boxWidth=" @BoxWidth;
            putq " excludeData=" @ExcludeData;
            putq " numberOfStandardDeviations=" @NumberOfStandardDeviations;
            putq " boundaryPercentile=" @BoundaryPercentile;
            putq " computeStandardErrorOfMean=" @StandardErrorOfMean;
            putq " usePooledVariances=" @PooledVariances;
            putq " labelPosition=" @LabelPosition;
            put ">" CR;
          finish:
            put "</Box>" CR;
            xdent;
        end;

        define event PLOT;
          start:
            ndent;
            put "<Plot";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " styleBy=" @StyleBy;
            putq " responseAxis=" @ResponseAxis;
            putq " breaksVisible=" @BreaksVisible;
            putq " bubbleSize=" @BubbleSize;
            putq " labelPosition=" @LabelPosition;
            putq " labelStatistic=" @LabelStatistic;
            putq " excludeData=" @ExcludeData;
            putq " displayInBack=" @DisplayInBack;
            put ">" CR;
          finish:
            put "</Plot>" CR;
            xdent;
        end;



        define event Contour;
          start:
            ndent;
            put "<Contour";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " styleBy=" @StyleBy;
            putq " contourStyle=" @ContourStyle;
            putq " numberOfLevels=" @NumberOfLevels;
            put ">" CR;
          finish:
            put "</Contour>" CR;
            xdent;
        end;



        define event Scatter3d;
          start:
            ndent;
            put "<Plot";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " styleBy=" @StyleBy;
            put ">" CR;
          finish:
            put "</Plot>" CR;
            xdent;
        end;



        define event MapChart;
          start:
            ndent;
            put "<Map";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " styleBy=" @StyleBy;
            put ">" CR;
          finish:
            put "</Map>" CR;
            xdent;
        end;


        define event NoDataFill;
          start:
            ndent;
            put "<NoDataFill ";
            putq "fillColor=" @FillColor;
            put ' fillType="Pattern"/>' CR;
            xdent;
        end;


        define event MapBar;
          start:
            ndent;
            put "<MapBar";
            putq " barShape=" @BarShape;
            putq " barWidth=" @BarWidth;
            put ">" CR;
          finish:
            put "</MapBar>" CR;
            xdent;
        end;


        define event Surface;
          start:
            style=GraphCharts;
            pure_style;
            ndent;
            put "<Surface";
            putq " name=" ref_id;
            putq " chartData=" @ChartDataID;
            putq " styleBy=" @StyleBy;
            put  ' isTransparent="true"' / if exists(TRANSPARENCY);
            putq " gridStyle=" @GridStyle;
            putq " surfaceContourStyle=" @SurfaceContourStyle;
            putq " fillUnderSurface=" @FillUnderSurface;
            putq " topColor=" @TopColor;
            putq " bottomColor=" @BottomColor;
            putq " surfaceStyle=" @SurfaceStyle;
            putq " numberOfLevels=" @NumberOfLevels;
            put ">" CR;
          finish:
            put "</Surface>" CR;
            xdent;
        end;

        define event DonutLabel;
          start:
            ndent;
            put "<DonutLabel";
            putq " textColor=" @TextColor;
            putq " fontSize=" @TextSize;
            putq " textVisible=" @TextVisible;
            putq " textBaselineAngle=" @TextBaselineAngle;
            putq " textCharacterAngle=" @TextCharacterAngle;
            putq " textAlign=" @TextHorizontalAlignment;
            putq " fontFamily=" @TextFont;
            putq " textDecoration=" @TextDecoration;
            putq " fontWeight=" @FontWeight;
            putq " fontStyle=" @FontStyle;
            put ">" @TextValue "</DonutLabel>" CR / if exists(@TextValue);
            break / if exists(@TextValue);
            put "/>" CR;
            xdent;
        end;


        define event ExplodeValues;
          start:
            ndent;
            put "<ExplodeValues>" CR;
          finish:
            put "</ExplodeValues>" CR;
            xdent;
        end;


        define event InvisibleValues;
          start:
            ndent;
            put "<InvisibleValues>" CR;
          finish:
            put "</InvisibleValues>" CR;
            xdent;
        end;



        define event ErrorBarFill;
          start:
            ndent;
            put "<ErrorBarFill";
            putq " fillColor=" @FillColor;
            putq " fillVisible=" @FillVisible;
            put "/>" NL;
            xdent;
        end;



        define event Interpolations;
          start:
            ndent;
            put "<Interpolations>" CR;
          finish:
            put "</Interpolations>" CR;
            xdent;
        end;



        define event Area;
          start:
            ndent;
            put "<Area";
            putq " fillColor=" @FillColor;
            putq " fillFile=" @FillFile;
            putq " fillType=" @FillType;
            put ">" CR;
          finish:
            put "</Area>" CR;
            xdent;
        end;



        define event OutlierSymbol;
          start:
            ndent;
            put "<OutlierSymbol";
            putq " markerSymbol=" @MarkerSymbol;
            putq " markerVisible=" @MarkerVisible;
            putq " markerSize=" @MarkerSize;
            putq " markerColor=" @MarkerColor;
            put "/>" NL;
            xdent;
        end;



        define event Needle;
          start:
            ndent;
            put "<Needle";
            putq " lineVisible=" @LineVisible;
            putq " lineStyle=" @LineStyle;
            putq " lineColor=" @LineColor;
            putq " lineThickness=" @LineThickness;
            put "/>" NL;
            xdent;
        end;



        define event Levels;
          start:
            ndent;
            put "<Levels>" CR;
          finish:
            put "</Levels>" CR;
            xdent;
        end;



        define event Level;
          start:
            ndent;
            put "<Level>";
            put  value;
          finish:
            put "</Level>" CR;
            xdent;
        end;



        define event GroupTickValues;
          start:
            ndent;
            put "<GroupTickValues>" CR;
          finish:
            put "</GroupTickValues>" CR;
            xdent;
        end;

        define event MidpointValues;
          start:
            ndent;
            put "<MidpointValues";
            putq " Formatted=" @Formatted;
            put ">" CR;
          finish:
            put "</MidpointValues>" CR;
            xdent;
        end;

        define event TickValues;
          start:
            ndent;
            put "<TickValues>" CR;
          finish:
            put "</TickValues>" CR;
            xdent;
        end;

        define event GraphTitles;
          start:
            ndent;
            put "<Titles>" CR;
          finish:
            put "</Titles>" CR;
            xdent;
        end;

        define event GraphTitle;
            ndent;
            put "<Title>" @TextValue "</Title>" CR / if exists(@TextValue);
            xdent;
        end;

        define event MapWKB;
          start:
            ndent;
            put "<MapWKB>";
            put value;
          finish:
            put "</MapWKB>" CR;
            xdent;
        end;


        define event graph_border;
        end;

        define event graph_actx_width;
           style=Graph;
           pure_style;
           set $outputwidth '640';
           set $outputwidth colwidth / if ^cmp(colwidth, '0');
           set $outputwidth outputwidth / if outputwidth;
        end;

        define event graph_actx_height;
           style=Graph;
           pure_style;
           set $outputheight '480';
           set $outputheight depth / if ^cmp(depth, '0');
           set $outputheight outputheight / if outputheight;
        end;


        define event graph_fxd_attributes;
            /*pure_style;*/
            style=Graph;
            trigger graph_actx_width;
            trigger graph_actx_height;
            trigger graph_style_inline;
            put  ' ALIGN="top" class="Graph">' CR;
            put '<PARAM NAME="ENCODING" VALUE=' / if exists(ENCODING);
            putq ENCODING / if exists(ENCODING);
            put  '>' CR / if exists(ENCODING);
        end;

        define event graph_style_inline;
            put 'style="';
            put ' width: ' prxchange('s/(\.|\d)$/\1px/', -1, $outputwidth) ';' / exists($outputwidth);
            put ' height: ' prxchange('s/(\.|\d)$/\1px/', -1, $outputheight) ';' / exists($outputheight);
            put ' background-color: ' background ';' / exists(background);
            /* goptions noborder */
            put ' border-width: 0px;' / cmp(borderwidth, '0px');
            put '"';
        end;

        define event graph_java_width;
           style=Graph;
           pure_style;
           set $outputwidth '640';
           set $outputwidth colwidth / if ^cmp(colwidth, '0');
           set $outputwidth outputwidth / if outputwidth;
        end;

        define event graph_java_height;
           style=Graph;
           pure_style;
           set $outputheight '480';
           set $outputheight depth / if ^cmp(depth, '0');
           set $outputheight outputheight / if outputheight;
        end;

        define event graph_java2_attribute;
           put '   document.writeln("';
           put NAME '=\"' VALUE '\"';
           put '");' CR;
        end;

        define event graph_java2_fxd_attributes;
           put '   document.writeln(">");' CR;
           trigger javascript finish;
        end;

        define event graph_attribute;
            put " " NAME;
            putq "=" VALUE CR;
        end;

        define event graph_parameter;
            putq "<PARAM NAME=" NAME;
            putq " VALUE=" VALUE;
            put ">" CR;
        end;

        define event graph_onload;
        end;

        define event graph_fillin_block;
        end;

        define event graph_fxd_parameters;
            putq '<PARAM NAME="BackColor" VALUE=' BACKGROUND;
            put ">" CR / exists(BACKGROUND );
        end;

        define event html3_center;
            start:
               put "<CENTER>" CR;
            finish:
               put "</CENTER>" CR;
        end;

        define event java_unsupported;
            put "Sorry, your browser does not support the applet tag." CR;
            putq 'The graph ' value ' cannot be displayed.' CR;
        end;

        define event activex_unsupported;
            put "Sorry, there was a problem with the Graph control or plug-in in your browser." CR;
            putq 'The graph ' value ' cannot be displayed.' CR;
        end;

        define event graph_description;
            putq value CR;
        end;

        define event activex_graph;
            start:
                put "<OBJECT " CR;
                putq " ID=" ref_id CR;
            finish:
                put "</OBJECT> " CR;
        end;

        define event java2_parameters;
                put  '<PARAM NAME="TYPE" VALUE="application/x-java-applet;version=1.6">' CR;
                put  '<PARAM NAME="ENCODING" VALUE=' / if exists(ENCODING);
                putq  ENCODING / if exists(ENCODING);
                put  '>' CR / if exists(ENCODING);
                put  '<PARAM NAME="SCRIPTABLE" VALUE="true">' CR;
        end;


        /* This event is to allow the xmldoc being driven by graph */
        /* to be able to write out an ODSTAG string to the HTML    */
        /* document. Do not remove this event unless you find out  */
        /* that EG does not need ODSTAG strings any more. --sandy  */

        define event odstag_eg;
            put "<!-- ODSTAG TYPE=" value ;
            put " ID=" anchor ;
            put  " -->" CR;
        end;

        define event java2_graph;
            start:
                pure_style;
                style=Graph;
                trigger ie_check / if !exists($ieCheckDone);
                trigger javascript start;
                put  '   document.writeln("<OBJECT");' CR;
                put  '   document.writeln(''';
                trigger graph_java_width;
                trigger graph_java_height;
                trigger graph_style_inline;
                put  ''');' CR;
                put  '   document.writeln("ALIGN=\"baseline\" class=\"Graph\"");' CR;
                put  'if (_ie == true) {' CR;
                put  '   document.writeln("CLASSID=\"clsid:8AD9C840-044E-11D1-B3E9-00805F499D93\"");' CR;
                do / if CODEBASE;
                put  '   document.writeln("CODEBASE=\"' CODEBASE '\"");' CR;
                else;
                put  '   if (_ie64 == true) {' CR;
                put  '      document.writeln("CODEBASE=\"http://javadl.sun.com/webapps/download/AutoDL?BundleId=47155#Version=1,6,0,0\"");' CR;
                put  '   }' CR;
                put  '   else {' CR;
                put  '      document.writeln("CODEBASE=\"http://java.sun.com/update/1.6.0/jinstall-6u24-windows-i586.cab#Version=1,6,0,0\"");' CR;
                put  '   }' CR;
                done;
                put  '}' CR;
                put  'else {' CR;
                put  '   document.writeln("CODEBASE=\"http://javadl.sun.com/webapps/download/GetFile/1.6.0_24-b07/windows-i586/xpiinstall.exe\"");' CR;
                put  '   document.writeln("TYPE=\"application/x-java-applet;version=1.6\"");' CR;
                put  '}' CR;

            finish:
                put  '</OBJECT>' CR;
            end;


        define event javascript;
         start:
             put '<script language="javascript" type="text/javascript">' CR;
             put "<!-- " CR;
         finish:
             put CR "//-->"  CR;
             put "</script>" CR CR;
             put '<noscript></noscript>' CR;               /* 508 */
        end;

    end;
    %end;  %mend;
    %tagsets_graph;
run;
